//李知青：TAA原理及OpenGL实现
//https://zhuanlan.zhihu.com/p/479530563

// 月光下的旅行: Vulkan TAA实现与细节
// https://qiutang98.github.io/post/%E5%9B%BE%E5%BD%A2%E7%A1%AC%E4%BB%B6api/vulkan-taa%E5%AE%9E%E7%8E%B0%E4%B8%8E%E7%BB%86%E8%8A%82/

//抖动
const vec2 Halton_2_3[8] = vec2[8](
    vec2(0.0f, -1.0f / 3.0f),
    vec2(-1.0f / 2.0f, 1.0f / 3.0f),
    vec2(1.0f / 2.0f, -7.0f / 9.0f),
    vec2(-3.0f / 4.0f, -1.0f / 9.0f),
    vec2(1.0f / 4.0f, 5.0f / 9.0f),
    vec2(-1.0f / 4.0f, -5.0f / 9.0f),
    vec2(3.0f / 4.0f, 1.0f / 9.0f),
    vec2(-7.0f / 8.0f, 7.0f / 9.0f)
);

#ifdef CPS_FSH

vec2 unTAAJitterTexcoord(vec2 uv){
    vec2 jitter = Halton_2_3[frameCounter % 8];	//-1 to 1
	jitter /= vec2(viewWidth, viewHeight);
    vec2 newUV = uv - jitter * TAA_JITTER_AMOUNT * 0.5;

    return newUV;
}

//屏幕空间位置转视口位置，反TAA抖动版本
vec4 unTAAJitterSPosToVPos(vec4 screenPos){
    // vec2 jitter = Halton_2_3[frameCounter % 8];	//-1 to 1
	// jitter /= vec2(viewWidth, viewHeight);

    screenPos.xy = unTAAJitterTexcoord(screenPos.xy);
    vec4 NDCPos = vec4(screenPos.xyz * 2.0 - 1.0, 1.0);
    vec4 clipPos = gbufferProjectionInverse * NDCPos;
    return vec4(clipPos.xyz / clipPos.w, 1.0); 
}

//视口位置转屏幕空间位置，反TAA抖动版本
vec4 unTAAJitterVPosToSPos(vec4 viewPos){
    vec4 clipPos = gbufferProjection * viewPos;
    vec3 NDCPos = clipPos.xyz / clipPos.w;
    vec4 screenPos = vec4(NDCPos.xyz*0.5 + 0.5, 1.0);

    vec2 jitter = Halton_2_3[frameCounter % 8];	//-1 to 1
    jitter /= vec2(viewWidth, viewHeight);
    screenPos.xy += jitter * TAA_JITTER_AMOUNT;

    return vec4(NDCPos.xyz*0.5 + 0.5, 1.0);
}

//上一帧所在位置
//参考了BSL Shader的代码
vec2 getPrePos(vec4 worldPos){
    vec3 cameraOffset = cameraPosition - previousCameraPosition;    //prevPos to nowPos
    vec4 prePos = worldPos + vec4(cameraOffset, 0.0);
	prePos = gbufferPreviousModelView * prePos;
	prePos = gbufferPreviousProjection * prePos;
	return prePos.xy / prePos.w * 0.5 + 0.5;
}

float Luminance(vec3 color){
    return 0.25 * color.r + 0.5 * color.g + 0.25 * color.b;
}

vec3 ToneMap(vec3 color){
    return color / (1 + Luminance(color));
}

vec3 UnToneMap(vec3 color){
    return color / (1 - Luminance(color));
}

vec3 clipAABB(vec3 nowColor, vec3 preColor){
    //累计周围颜色值及平方值
    vec2 deltaRes = vec2(1.0 / viewWidth, 1.0 / viewHeight);
    vec3 m1 = vec3(0), m2 = vec3(0);

    for(int i = -1; i <= 1; i++){
    for(int j = -1; j <= 1; j++){
        vec2 newUV = texcoord.st + deltaRes * vec2(i, j);
        vec3 C = RGB2YCoCgR(ToneMap(texture(colortex0, newUV).rgb));
        m1 += C;
        m2 += C * C;
    }
    }

    //计算均值和标准差，以确定包围盒大小
    const float TAA_variance_clip_gamma = 1.0;
    vec3 aabbMin = nowColor, aabbMax = nowColor;
    const int N = 9;
    vec3 mu = m1 / N;
    vec3 sigma = sqrt(abs(m2 / N - mu * mu));
    aabbMin = mu - TAA_variance_clip_gamma * sigma;
    aabbMax = mu + TAA_variance_clip_gamma * sigma;

    //根据包围盒大小进行约束
    vec3 p_clip = 0.5 * (aabbMax + aabbMin);
    vec3 e_clip = 0.5 * (aabbMax - aabbMin);

    vec3 v_clip = preColor - p_clip;
    vec3 v_unit = v_clip.xyz / e_clip;
    vec3 a_unit = abs(v_unit);
    float ma_unit = max(a_unit.x, max(a_unit.y, a_unit.z));

    if (ma_unit > 1.0)
        return p_clip + v_clip / ma_unit;
    else
        return preColor;
}



float getBlendFactor(vec2 velocity, vec3 preColor, vec3 nowColor){
    float blendFactor = 1.0;
    float lDepth = linearDepthToExpDepth(texture(depthtex1, texcoord.st).r);
    float lVelocity = length(velocity);
    float LDmLV = lDepth * lVelocity;
    blendFactor *= LDmLV * 0.5 + clamp01(0.5 / LDmLV + 0.000001) * 0.1666;

    float lumPreColor = getLuminance(preColor);
    float lumNowColor = getLuminance(nowColor);
    float lumDiff = 1.0 - abs(lumNowColor - lumPreColor) / (max(lumNowColor, lumPreColor) + 0.3);
    blendFactor *= lumDiff;

    // blendFactor = 0.05;
    return clamp(blendFactor, 0.0, 0.1);
}

void TAA(inout vec3 nowColor){
    vec2 velocity = texture(colortex3, getClosestOffset(texcoord, 1.0).st).rg;
    vec2 offsetUV = clamp01(texcoord.st - velocity);
    // vec3 preColor = texture(colortex2, offsetUV).rgb;
    // vec3 preColor = catmullRom(colortex2, offsetUV).rgb;
    vec3 preColor = max(BLACK, catmullRom5(colortex2, offsetUV, 0.8).rgb);
    
    nowColor = RGB2YCoCgR(ToneMap(nowColor));
    preColor = RGB2YCoCgR(ToneMap(preColor));

    preColor = clipAABB(nowColor, preColor);

    preColor = UnToneMap(YCoCgR2RGB(preColor));
    nowColor = UnToneMap(YCoCgR2RGB(nowColor));

    // float blendFactor = 0.05;
    float blendFactor = getBlendFactor(velocity, preColor, nowColor);

    nowColor = mix(preColor, nowColor, blendFactor);
}

#endif