// 未名客：【实战】从零实现一套完整单次大气散射
// https://zhuanlan.zhihu.com/p/237502022
// 感谢 冷夜 大佬在实现过程中给予的帮助


vec3 ScatteringR = RayleighSct * RayleighScatterCoef;
vec3 ScatteringM = MieSct * MieScatterCoef;

vec3 ExtinctionR = RayleighSct * RayleighExtinctionCoef;
vec3 ExtinctionM = MieSct * MieExtinctionCoef;


float getHeigth(vec3 pos){
    return length(pos - planetCenter) - earth_r;
}

// Atmospheric Density Ratio (大气密度比例方程)
// exp(-h/H), H=vec2(R 8500, M 1200)
vec2 calculateAtmosphereDensity(vec3 pos){
    float height = getHeigth(pos);
    vec2 density = min(exp(-(height.xx / densityScale_h)), vec2(1.0));
    // density = pow(density, vec2(0.5));
    // density *= 2.0;
    // vec2 density = exp(-(height.xx / densityScale_h));
    
    return density;
}

float calculateOzoneDensity(vec3 pos){
    float height = getHeigth(pos);
    return max(0, (1.0 - abs(height - 25000.0)) / 15000.0);
}

// D CP，x: Rayleigh，y: Mie
void DensityCP(vec3 pos, vec3 sunDir, out vec2 dcp, out float doz, float sampleCountCP){
    vec3 rayStart = pos;
	vec3 rayDir = sunDir;

    // 求与大气球的交点距离
    float intersection = RaySphereIntersection(rayStart, rayDir, planetCenter, earth_r + atmosphere_h).y;
    vec3 rayEnd = rayStart + rayDir * intersection; // 交点位置=起点+方向*交点距离
    const float stepCount = sampleCountCP;
	vec3 stepVec = (rayEnd - rayStart) / stepCount;
	float stepSize = length(stepVec);
    
    rayStart += stepVec * 0.5;

	dcp = vec2(0.0);
    vec2 preDcp = calculateAtmosphereDensity(rayStart);
    doz = 0.0;
    float preDoz = calculateOzoneDensity(rayStart);

    for (float s = 1.0; s < stepCount; s += 1.0){
		vec3 nowPos = rayStart + stepVec * s;

        vec2 nowDcp = calculateAtmosphereDensity(nowPos);
        dcp += (nowDcp + preDcp) /* * stepSize * 0.5 */;

        float nowDoz = calculateOzoneDensity(nowPos);
        doz += (nowDoz + preDoz)/* * stepSize * 0.5*/;

        preDcp = nowDcp;
        preDoz = nowDoz;
	}
    dcp *= stepSize * 0.5;
    doz *= stepSize * 0.5;
    // for (float s = 0.5; s < stepCount; s += 1.0){
	// 	vec3 nowPos = rayStart + stepVec * s;
    //     nowPos += stepVec * rand3_1(nowPos);

    //     vec2 nowDcp = calculateAtmosphereDensity(nowPos);
    //     dcp += nowDcp * stepSize;

    //     float nowDoz = calculateOzoneDensity(nowPos);
    //     doz += nowDoz * stepSize;
	// }

}

void getAtmosphereDensity(vec3 pos, vec3 sunDir, out vec2 dcp, out vec2 dpa, out float doz1, out float doz2, float sampleCountCP){
    dpa = calculateAtmosphereDensity(pos);
    doz2 = calculateOzoneDensity(pos);
    DensityCP(pos, sunDir, dcp, doz1, sampleCountCP);
}

// Transmittance Function (投射率方程)
// T(CP)*T(PA) = exp(-(D(CP+PA) * β(x:R,y:M)))
// return T(CP)*T(PA) * localDensity
void localInscattering(vec2 localDensity, vec2 dpa, vec2 dcp, float doz1, float doz2, out vec3 InscatterR, out vec3 InscatterM){
    vec2 dcpa = dpa + dcp;

	vec3 Tr = dcpa.x * ExtinctionR;
	vec3 Tm = dcpa.y * ExtinctionM;
    vec3 Am = dcpa.y * vec3(4.4) * 1e-6;
    vec3 Ao = (doz1 + doz2) * vec3(0.650, 1.881, 0.085) * 1e-6;

    vec3 extinction = exp(-(Tr + Tm + Am + Ao));

	InscatterR = localDensity.x * extinction;
	InscatterM = localDensity.y * extinction;
}

// Phase Function(相位函数) 
// R: 3/(16 * PI) * (1 + (cosθ)^2)
// M: (1-g2) / (4 * PI * ( 1 + g2 - 2 * g * cosθ)^(3/2))
void ApplyPhaseFunction(inout vec3 scatterR, inout vec3 scatterM, float cosAngle) {
	// r
	float phase = (3.0 / (16.0 * PI)) * (1 + (cosAngle * cosAngle));
	scatterR *= phase;

	// m
	float g = MieG;
	float g2 = g * g;
	phase = (1.0 / (4.0 * PI)) * ((3.0 * (1.0 - g2)) / (2.0 * (2.0 + g2))) * ((1 + cosAngle * cosAngle) / (pow((1 + g2 - 2 * g * cosAngle), 3.0 / 2.0)));
    // phase = (1 - g2) / (4.0 * PI * pow((1 + g2 - 2 * g * cosAngle), 3.0 / 2.0));

	scatterM *= phase;
}

vec3 AtmosphericScattering(vec3 worldDir, vec3 sunDir, float sampleCountPA, float sampleCountCP){
    vec2 dcp = vec2(0.0);
    vec2 dpa = vec2(0.0);
    vec2 preLocalDensity = vec2(0.0);
    vec2 localDensity = vec2(0.0);
    float doz1 = 0.0;   // 臭氧密度
    float doz2 = 0.0;
    float preDoz2 = 0.0;
    float nowDoz2 = 0.0;

    vec3 scatterR = vec3(0.0);
    vec3 scatterM = vec3(0.0);
    vec3 prevLocalInscatterR = vec3(0.0);   
    vec3 prevLocalInscatterM = vec3(0.0); 
    vec3 localInscatterR = vec3(0.0);
    vec3 localInscatterM = vec3(0.0);

    vec3 rayStart = vec3(0.0, camera_h, 0.0);
    vec3 rayDir = worldDir;
    // if(rayDir.y < -0.1) {
    //     rayDir.y = -0.1;
    // }
    // rayDir = normalize(rayDir);

    // 求交点距离(与大气球的内交点，与地球的外交点)
    float intersection = RaySphereIntersection(rayStart, rayDir, planetCenter, earth_r + atmosphere_h).y;
    float intersection_e = RaySphereIntersection(rayStart, rayDir, planetCenter, earth_r).x;
    if(intersection_e > 0.0) 
        intersection = min(intersection_e, intersection);

    float sampleCount = sampleCountPA;
    vec3 stepVec = rayDir * (intersection / sampleCount);     //方向 *（交点距离 / 步进次数）
    float stepSize = length(stepVec);
    rayStart += stepVec * 0.5;

    // 起点处的 大气密度 和 内散射
    getAtmosphereDensity(rayStart, sunDir, dcp, preLocalDensity, doz1, preDoz2, sampleCountCP);
    // 梯形积分版
    localInscattering(preLocalDensity, dpa, dcp, doz1, doz2, prevLocalInscatterR, prevLocalInscatterM);
    for(float s = 1.0; s < sampleCount; s += 1.0){
        vec3 nowPos = rayStart + stepVec * s;
        // if(getHeigth(nowPos) < 0.0) nowPos.y = 0.0;

        // 更新后端点大气密度、累计pa的大气密度、累计pa的臭氧密度、更新后端点的内散射
        getAtmosphereDensity(nowPos, sunDir, dcp, localDensity, doz1, nowDoz2, sampleCountCP);
        dpa += (localDensity + preLocalDensity) * stepSize * 0.5;
        doz2 += (nowDoz2 + preDoz2) * stepSize * 0.5;
        localInscattering(localDensity, dpa, dcp, doz1, doz2, localInscatterR, localInscatterM);

        // 累计内散射
        scatterR += (prevLocalInscatterR + localInscatterR) /** stepSize * 0.5*/;
        scatterM += (prevLocalInscatterM + localInscatterM) /** stepSize * 0.5*/;

        prevLocalInscatterR = localInscatterR;
        prevLocalInscatterM = localInscatterM;
        preLocalDensity = localDensity;
        preDoz2 = nowDoz2;
    }
    scatterR *= stepSize * 0.5;
    scatterM *= stepSize * 0.5;
    // 噪声版
    // for(float s = 0.5; s < sampleCount; s += 1.0){
    //     vec3 nowPos = rayStart + stepVec * s;
    //     nowPos += stepVec * rand3_1(nowPos);
    //     // if(getHeigth(nowPos) < 0.0) nowPos.y = 0.0;

    //     getAtmosphereDensity(nowPos, sunDir, dcp, localDensity, doz1, nowDoz2);
    //     dpa += localDensity * stepSize;
    //     doz2 += nowDoz2 * stepSize;
    //     localInscattering(localDensity, dpa, dcp, doz1, doz2, localInscatterR, localInscatterM);

    //     // 累计内散射
    //     scatterR += localInscatterR * stepSize;
    //     scatterM += localInscatterM * stepSize;
    // }

	ApplyPhaseFunction(scatterR, scatterM, dot(rayDir, sunDir));
    vec3 ASI = IncomingLight.rgb /*+ 50.0 * mix(BLACK, sunColor, dot(rayDir, sunDir) > 0.999 ? 1.0 : 0.0)*/;
	vec3 lightInscatter = (scatterR * ScatteringR + scatterM * ScatteringM) * ASI;
    lightInscatter = mix(lightInscatter, vec3(getLuminance(lightInscatter)), rainStrength * 0.8);

	return lightInscatter;
}






// AKG4e3: 实时大气散射渲染实战
// https://zhuanlan.zhihu.com/p/595576594

const vec3 RayleighSigma = vec3(5.802, 13.558, 33.1) * 1e-6;
const vec3 MieSigma = vec3(3.96) * 1e-6;
const vec2 H = vec2(8500.0, 1200.0);

float getRho(float h, float H){
    // return min(exp(-(h / H)), 1.0);
    return exp(-(h / H));
}

vec3 RayleighCoefficient(float H, float h) {
    return RayleighSigma * getRho(h, H);
}

float RayleighPhase(float cosTheta){
    return (3.0 / (16.0 * PI)) * (1.0 + cosTheta * cosTheta);
}

vec3 MieCoefficient(float H, float h){
    return MieSigma * getRho(h, H);
}

float MiePhase(float cosTheta, float g){
    float g2 = g * g;
    float a = 3.0 / (8.0 * PI);
    float b = (1.0 - g2) / (2.0 + g2);
    float c = 1.0 + cosTheta * cosTheta;
    float d = pow(1.0 + g2 - 2.0 * g * cosTheta, 1.5);
    return a * b * (c / d);
}

vec3 Scattering(vec3 p, vec3 inDir, vec3 outDir){
    float cosTheta = dot(inDir, outDir);
    float h = getHeigth(p);

    vec3 rayleigh = RayleighCoefficient(H.x, h) * RayleighPhase(cosTheta);
    vec3 mie = MieCoefficient(H.y, h) * MiePhase(cosTheta, MieG);

    return rayleigh + mie;
}

vec3 MieAbsorption(float H, float h){
    const vec3 sigma = vec3(4.4) * 1e-6;
    return sigma * getRho(h, H);
}

vec3 OzoneAbsorption(float h){
    const vec3 sigma = vec3(0.650, 1.881, 0.085) * 1e-6;
    const float center = 25000;
    const float width = 15000;
    float rho = max(0, (1.0 - abs(h - center)) / width);
    return sigma * rho;
}

vec3 Transmittance(vec3 p1, vec3 p2){
    const int n = 10;

    vec3 dir = normalize(p2 - p1);
    float dist = length(p2 - p1);
    float ds = dist / float(n);
    vec3 sum = vec3(0.0);
    vec3 p = p1 + dir * ds * 0.5;

    for(int i = 0; i < n; i++){
        float h = getHeigth(p);

        vec3 scattering = RayleighCoefficient(H.x, h) + MieCoefficient(H.y, h);
        vec3 absorption = OzoneAbsorption(h) + MieAbsorption(H.y, h);
        vec3 extinction = scattering + absorption;

        sum += extinction * ds;
        p += dir * ds;
    }

    return exp(-sum);
}

vec3 AtmosphericScattering1(vec3 worldDir, vec3 sunDir){
    vec3 color = vec3(0.0);
    
    int n = 10;
    vec3 rayStart = vec3(0.0, camera_h, 0.0);
    vec3 rayDir = worldDir;
    float d_e2a = RaySphereIntersection(rayStart, rayDir, planetCenter, earth_r + atmosphere_h).y;
    // float d_2ea = RaySphereIntersection(rayStart, rayDir, planetCenter, earth_r).x;
    // if(d_e2a > 0.0) 
    //     d_e2a = min(d_e2a, d_2ea);
    float ds = d_e2a / n;
    vec3 p1 = rayStart + rayDir * ds * 0.5;

    for(int i = 0; i < n; i++){
        float d_2sp = RaySphereIntersection(p1, sunDir, planetCenter, earth_r + atmosphere_h).y;
        vec3 p2 = p1 + sunDir * d_2sp;

        vec3 t1 = Transmittance(p2, p1);
        vec3 s = Scattering(p1, rayDir, sunDir);
        vec3 t2 = Transmittance(p1, rayStart);

        vec3 inScattering = t1 * s * t2 * ds * vec3(1.0) * 4.0;
        color += inScattering;
        p1 += rayDir * ds;
    }

	return color;
}




vec3 Transmittance1(vec3 startPos, vec3 rayDir, float stepCount){
    float intersection = RaySphereIntersection(startPos, rayDir, planetCenter, earth_r + atmosphere_h).y;
    vec3 stepVec = rayDir * (intersection / stepCount);
    float stepSize = length(stepVec);

    vec2 density = vec2(0.0);
    startPos +=  0.5 * stepVec;
    vec2 preDensity = calculateAtmosphereDensity(startPos);
    for (float s = 1.0; s < stepCount; s += 1.0){
		vec3 nowPos = startPos + stepVec * s;

        vec2 nowDensity = calculateAtmosphereDensity(nowPos);
        density += (nowDensity + preDensity) * stepSize * 0.5;

        preDensity = nowDensity;
	}

    vec3 Tr = density.x * ExtinctionR;
    vec3 Tm = density.y * ExtinctionM;

    return exp(-(Tr + Tm)) * IncomingLight;
}

vec3 Transmittance2(vec3 startPos, vec3 endPos, float stepCount, vec3 oriColor){
    vec3 direction = normalize(endPos - startPos);
    float dist = length(endPos - startPos);
    float stepSize = dist / stepCount;
    vec3 stepVec = direction * stepSize;

    vec2 density = vec2(0.0);
    startPos +=  0.5 * stepVec;
    vec2 preDensity = calculateAtmosphereDensity(startPos);
    for (float s = 1.0; s < stepCount; s += 1.0){
		vec3 nowPos = startPos + stepVec * s;

        vec2 nowDensity = calculateAtmosphereDensity(nowPos);
        density += (nowDensity + preDensity) * stepSize * 0.5;

        preDensity = nowDensity;
	}

    vec3 Tr = density.x * ExtinctionR;
    vec3 Tm = density.y * ExtinctionM;

    return exp(-(Tr + Tm)) * oriColor;
}


