vec3 drawSun(vec3 worldDir){
    float wds = dot(worldDir, sunWorldDir);
    float mixAmount = wds > 0.99985 ? 1.0 : 0.0; // 半径
    vec3 color = sunColor * 100.0;
    if(isEyeInWater == 1) color *= (0.01 * eyeBrightnessSmooth.y / 240.0 + 0.01);
    return mix(BLACK, color, mixAmount); // 颜色*亮度
}

// piyushslayer: Happy 2020!
// https://www.shadertoy.com/view/tt3GRN
vec3 drawMoon(vec3 worldDir){
    float moon = smoothstep(.030, .025, length(worldDir - moonWorldDir));
    mat3 offsetMat = rotate3D(vec3(1.0, 0.0, 0.0), _2PI * -0.00150) * rotate3D(vec3(0.0, 0.0, 1.0), _2PI * 0.00) * rotate3D(vec3(0.0, 1.0, 0.0), _2PI * 0.00150);
    moon *= 1.0 - smoothstep(.035, .030, length(offsetMat * worldDir - moonWorldDir));
    vec3 moonColor = nightColor * moon * 1.5;
    return moonColor;
}


// a_codecat：shader实现星空效果
// https://blog.csdn.net/a_codecat/article/details/127600739
vec3 drawStars(vec3 worldDir){
    vec3 uv = worldDir;
    uv *= 50;
    vec3 ipos = floor(uv);
    vec3 fpos = fract(uv);
    vec3 targetPoint = hash33(ipos + sin(frameTimeCounter * 0.5) * 0.00005);

    // 判断条件
    float dist = length(fpos - targetPoint);
    float size = 0.075;
    float isStar = 1.0 - step(size, dist);

    return nightColor * isStar;
}

vec3 drawCelestial(vec3 worldDir, float transmittance){
    vec3 celestialColor = BLACK;
    if(getLuminance(sunColor) > 0.0) celestialColor += drawSun(worldDir);
    if(getLuminance(nightColor) > 0.0) {
        celestialColor += drawStars(worldDir);
        celestialColor += drawMoon(worldDir);
    }
    
    float mixAmount = clamp01((worldDir.y + 0.01) * 4 - (1.0 - transmittance) * 0.0);   // 遮罩
    mixAmount *= pow(transmittance, 10.0);
    mixAmount *= 1.0 - rainStrength;

    return mix(BLACK, celestialColor, mixAmount);
}