// 体积云实时渲染光照简单原理
// https://zhuanlan.zhihu.com/p/629189750
 
float getCloudNoise(vec3 pos){
    return noisetex2_3(pos);
    // return simplex3d(pos * 0.4);
}

float getCloudDensity(vec3 worldPos) {
    vec3 coord = worldPos;
    vec3 t = vec3(frameTimeCounter * 0.015);
    coord *= 0.00115;
    coord.xz += t.xz;
    
#define CLOUD_M1
#ifdef CLOUD_M1
    float n  = getCloudNoise(coord + t)                         * 0.5;      coord *= 2.25;
          n += getCloudNoise(coord - t * 0.3)                   * 0.25;     coord *= 2.26;
          n += getCloudNoise(coord - t * 0.3 * 0.3)             * 0.125;    coord *= 2.27;
          n += getCloudNoise(coord - t * 0.3 * 0.3 * 0.3)       * 0.0625;   coord *= 2.28;
          n += getCloudNoise(coord - t * 0.3 * 0.3 * 0.3 * 0.3) * 0.03125;  coord *= 2.29;
          n += getCloudNoise(coord + t * 0.3 * 0.3 * 0.3 * 0.3 * 0.3) * 0.015625;
    float h = smoothstep(cloudH - cloudT / 2.0, cloudH + cloudT / 2.0, worldPos.y);
    // h = 1.0 - h;
    return pow(smoothstep(0.055 + 0.2 * h - 0.5 * rainStrength, 0.5, n * 1.15), 0.55) * 0.0055;
#endif

#ifdef CLOUD_M2
    coord.xz = floor(coord.xz);
    float n = getCloudNoise(vec3(coord.xz, 0.0));
    return n > 0.2 ? 0.0045 : 0.0;
#endif
    // return n * 0.055;
}

float hgPhase(float g, float cosTheta){
    float numer = 1.0f - g * g;
    float denom = 1.0f + g * g + 2.0f * g * cosTheta;
    return numer / (4.0f * PI * denom * sqrt(denom));
}

float dualLobPhase(float g0, float g1, float w, float cosTheta){
    return mix(hgPhase(g0, cosTheta), hgPhase(g1, cosTheta), w);
}

float BearsPowder(float d){
    return max(pow(d, 4.2) * exp(-d * 9), 0.0);
}

float getSunVisibility(float numSteps, vec3 pos, vec3 dir, float maxLen){
    vec3 stepVec = dir * maxLen / numSteps;
    float stepSize = length(stepVec);
    float sunVisibility = 1.0;
    for(int i = 0; i < numSteps + 0.1; ++i){
        vec3 nowPos = pos + stepVec * i;
        float density = getCloudDensity(nowPos);
        sunVisibility *= exp(-density * stepSize);
    }
    return min(1.0, sunVisibility);
}

void cloudRayMarching(vec3 startPos, vec3 worldDir, vec3 sunWorldDir, vec3 sunColor, const float quality, out vec4 cloudColor){
    if(dot(worldDir, upWorldDir) < 0.00){
        cloudColor = vec4(0.0, 0.0, 0.0, 1.0);
        return;
    }


    const float numSteps = quality;
    vec2 intersection = intersectAABB_Y(startPos, worldDir, cloudH, cloudT); 
    vec3 endPos = startPos + intersection.y * worldDir;
    startPos += intersection.x * worldDir;
    vec3 stepVec = (endPos - startPos) / numSteps;
    float stepSize = length(stepVec);

    
    float transmittance = 1.0f;
    vec3 scattering = vec3(0.0f);

    for(int i = 0; i < numSteps; ++i){
        vec3 nowPos = startPos + stepVec * i;
        nowPos += BlueNoise(texcoord * 2.0, 64.0) * stepVec * 0.8;
        // nowPos += rand2_1(texcoord + sin(frameTimeCounter)) * stepVec * 0.5 ;
        float worldDis = length(nowPos);
        if(worldDis > 12000) break;

        float density = getCloudDensity(nowPos);

        if(density > 0.00){
            float stepTransmittance = exp(-density * stepSize) * 1.0;
            transmittance *= pow(stepTransmittance, 1.00);



            float sunPhase = dualLobPhase(0.5, -0.5, 0.35, dot(worldDir, -sunWorldDir));    // 原 0.20

            float sunVisibility = 1.0f;
            float intersection1 = min(intersectAABB_Y(nowPos, sunWorldDir, cloudH, cloudT).y, 250.0); 
            sunVisibility *= getSunVisibility(2.0, nowPos, sunWorldDir, intersection1);
            // float intersection2 = min(intersectAABB_Y(nowPos, upWorldDir, cloudH, cloudT).y, 250.0); 
            // sunVisibility *= getSunVisibility(2.0, nowPos, upWorldDir, intersection2);
            sunVisibility = min(1.0, sunVisibility);
            sunVisibility = pow(sunVisibility, 0.5);
            
            vec3 stepScattering = sunColor * sunPhase * sunVisibility;
            
            vec3 sigmaS = vec3(density);
            const float sigmaA = 0.0;
            vec3 sigmaE = max(vec3(1e-8f), sigmaA + sigmaS);
            vec3 sactterLitStep = stepScattering * sigmaS;
            sactterLitStep = transmittance * (sactterLitStep - sactterLitStep * stepTransmittance);
            sactterLitStep /= sigmaE;
            scattering += sactterLitStep; 
        }
    }

    // float wdu = clamp01(pow(dot(worldDir, upWorldDir), 0.65));
    cloudColor = vec4(scattering, transmittance);
    // color = mix(color, color * pow(transmittance, 1.0) + scattering * 3.25, mix(wdu, 1.0, 0.2));
    
    return;
}

vec3 getVolumetricClouds(vec3 color, float depth, vec3 worldDir, vec3 normal, out float transmittance){
    vec2 cloudUV = texcoord * pow(0.5, 1.0) + vec2(0.0, 0.0);
    vec2 closestUV = cloudUV;
    float closestDepth = 1.0;

    float cloudDepth = texture(depthtex1, cloudUV * 2.0).r;
    if(cloudDepth == 1.0){
        closestUV = cloudUV;
    }else{
        for(int i = 0; i <= 4; ++i){
            vec2 newUV = cloudUV + cornerOffset[i] / texResolution;
            cloudDepth = texture(depthtex1, newUV * 2.0).r;
            if(cloudDepth == 1.0){
                closestUV = newUV;
                break;
            }  
        }
    }

    // Bilateral test
    // float ld = expDepthToLinearDepth(depth);
    // float w = 0.0;
    // float r = 0.5;
    // float q = 2.0;
    // float d = 2.0 * r / q;
    // vec4 c = vec4(BLACK, 0.0);
    // vec4 cc = texture(colortex4, cloudUV);
    // for(float i = -r; i < r + 0.1; i += d){
    // for(float j = -r; j < r + 0.1; j += d){
    //     vec2 offset = vec2(i, j) / texResolution;
    //     vec2 nuv = cloudUV + offset;
    //     float nld = expDepthToLinearDepth(texture(colortex5, nuv).r);
    //     vec3 nn = normalize(normalDecode(texture(colortex1, nuv).ba));
    //     vec4 nc = texture(colortex4, nuv);
    //     float weight = 1.0;
    //     // float weight = clamp(1.0f - abs(ld - nld) / 2.0f, 0.0f, 1.0f);
    //         // weight *= max(0.0f, dot(nn, normal) * 2.0f - 1.0f);
    //         weight *= clamp(1.0 - distance(cc, nc), 0.0, 1.0);
    //     if(nld < 1.0) weight *= 0.0;
    //     c += nc * weight;
    //     w += weight;
    // }
    // }
    // c /= max(0.001, w);

    vec4 cloudColor = texture(colortex4, closestUV);
    // vec4 cloudColor = c;
    vec3 scattering = cloudColor.rgb;
    transmittance = cloudColor.a;

    float wdu = smoothstep(0.0, 1.0, pow(dot(worldDir, upWorldDir), 0.45));
    color = mix(color, color * pow(transmittance, 1.0) + scattering * 2.5, mix(wdu, 1.0, 0.2));

    return color;
}