// 感谢实现过程中 GeForceLegend 提供的帮助

// D CP，x: Rayleigh，y: Mie
vec2 DensityCP1(vec3 pos, vec3 sunDir){
    vec3 rayStart = pos;
	vec3 rayDir = sunDir;

    float intersection = RaySphereIntersection(rayStart, rayDir, planetCenter, earth_r + atmosphere_h).y;
    vec3 rayEnd = rayStart + rayDir * intersection;
    const float stepCount = 5.0;
	vec3 stepVec = (rayEnd - rayStart) / stepCount;
	float stepSize = length(stepVec);
    
    rayStart += stepVec * 0.5;

	vec2 dcp = vec2(0.0);
    vec2 preDcp = calculateAtmosphereDensity(rayStart);

    for (float s = 1.0; s < stepCount; s += 1.0){
		vec3 nowPos = rayStart + stepVec * s;

        vec2 nowDcp = calculateAtmosphereDensity(nowPos);
        dcp += (nowDcp + preDcp) * stepSize * 0.5;

        preDcp = nowDcp;
	}
    return dcp;
}

void localInscattering1(vec2 localDensity, vec2 dpa, vec2 dcp, out vec3 InscatterR, out vec3 InscatterM){
    vec2 dcpa = dpa + dcp;

	vec3 Tr = dcpa.x * ExtinctionR;
	vec3 Tm = dcpa.y * ExtinctionM;

    vec3 extinction = exp(-(Tr + Tm));

	InscatterR = localDensity.x * extinction;
	InscatterM = localDensity.y * extinction;
}

vec3 volumetricLighting(vec2 vlUV, vec3 worldPos, vec3 sunDir){
    float sampleCount = 10.0 / clamp(60.0 / length(worldPos), 1.0, 5.0);
    vec3 worldDir = normalize(worldPos);
    float worldDis = length(worldPos);

    float stepSize = worldDis / sampleCount;
    // float stepSize = (far - near) / sampleCount;
    vec3 stepVec = worldDir * stepSize;

    vec3 startWorldPos = vec3(0.0, 0.0, 0.0);
    startWorldPos += worldDir * near;

    // float s = 0.0;
    // float c = 0.0;
    vec2 dpa = vec2(0.0);
    vec3 scatterR = vec3(0.0);
    vec3 scatterM = vec3(0.0);
    for(float i = 0; i < sampleCount; ++i){
        vec3 nowWorldPos = startWorldPos + stepVec * i;
        nowWorldPos += BlueNoise(texcoord, 64.0) * stepVec * 1.0;
        vec3 nowCameraPos = nowWorldPos + vec3(0.0, camera_h, 0.0);

        if(length(nowWorldPos) > min(worldDis, far)) break;

        vec3 shadowPos = getShadowPos(vec4(nowWorldPos, 1.0)).xyz;
        float shade = texture(shadowtex1, shadowPos);

        vec2 nDpa = calculateAtmosphereDensity(nowCameraPos);
        dpa += nDpa * stepSize;
        if(shade > 0.5){
            vec2 dcp = DensityCP1(nowCameraPos, sunDir);
            vec3 nScatterR, nScatterM;
            localInscattering1(nDpa, dpa, dcp, nScatterR, nScatterM);
            scatterR += nScatterR /** stepSize*/;
            scatterM += nScatterM /** stepSize*/;
        }
        // s += shade;
        // ++c;
    }
    scatterR *= stepSize;
    scatterM *= stepSize;

    ApplyPhaseFunction(scatterR, scatterM, dot(worldDir, sunDir));
    vec3 lightInscatter = (scatterR * ScatteringR + scatterM * ScatteringM) * IncomingLight ;
    // vec3 Tpa = exp(-(dpa.x * ExtinctionR + dpa.y * ExtinctionM)) * color;

    // color = pow(vec3(s / sampleCount), vec3(2.2));
    // color = lightInscatter + Tpa;
    lightInscatter *= (6000.0 * (1.0 + 0.5 * rainStrength) / far);  // 原 6000.0
    lightInscatter = mix(lightInscatter, vec3(getLuminance(lightInscatter)), rainStrength * 0.8);
    return lightInscatter;
}

vec3 getVolumetricLighting(vec3 color, float depth, vec3 worldPos){
    vec2 vlUV = texcoord * pow(0.5, 1.0) + vec2(0.5, 0.5);
    vec2 closestUV = vlUV;
    float closestDepth = 1.0;

    // for(int i = -1; i <= 1; ++i){
    // for(int j = -1; j <= 1; ++j){
    //     vec2 newUV = vlUV + 1.0 * vec2(i, j) / texResolution;
    //     float vlDepth = texture(colortex5, newUV).r;
    //     float diffDepth = abs(depth - vlDepth);
    //     if(diffDepth < closestDepth){
    //         closestDepth = diffDepth;
    //         closestUV = newUV;
    //     }
    // }    
    // }

    for(int i = 0; i < 4; ++i){
        vec2 newUV = vlUV + cornerOffset[i] / texResolution;
        float vlDepth = texture(depthtex1, (newUV - vec2(0.5, 0.5)) * exp2(1.0)).r;
        // float vlDepth = texture(colortex5, newUV).r;
        float diffDepth = abs(depth - vlDepth);
        if(diffDepth < closestDepth){
            closestDepth = diffDepth;
            closestUV = newUV;
        }
    }
    vec3 VLColor = texture(colortex4, closestUV).rgb;

    // 衰减
    vec3 cameraPos = vec3(0.0, camera_h, 0.0);
    color = Transmittance2(cameraPos, worldPos + cameraPos, 5.0, color);
    color += VLColor;

    return color;
}