const vec2 cornerOffset[5] = vec2[](
    vec2(0.0, 0.0),
    vec2(1.0, 1.0),
    vec2(1.0, -1.0),
    vec2(-1.0, 1.0),
    vec2(-1.0, -1.0)
);

//AKG4e3 : 高质量泛光（bloom）从理论到实战
//https://zhuanlan.zhihu.com/p/525500877

//五组2*2滤波
const vec2 FilterOffset5_22[4] = vec2[4](
    vec2(0.0, 1.0),
    vec2(1.0, 0.0),
    vec2(0.0, -1.0),
    vec2(-1.0, 0.0)
);

vec3 Filter22(sampler2D text, vec2 uv, float scale, float lod){
    vec2 deltaRes = vec2(1.0 / viewWidth, 1.0 / viewHeight);
    deltaRes *= scale;
    vec3 color = vec3(0.0);

    for(int i = 0; i < 4; i++){
        vec2 offset = FilterOffset5_22[i] * deltaRes;
        color += textureLod(text, uv + offset, lod).rgb;
    }

    color /= 4.0;
    
    return color;
}

vec3 Filter5_22(sampler2D text, vec2 uv, float scale, float lod){
    // if(outScreen(uv, scale / 2.0 + 0.1)) 
    //     return vec3(0.0);

    vec2 deltaRes = vec2(1.0 / viewWidth, 1.0 / viewHeight);
    deltaRes *= scale;
    vec3 color = vec3(0.0);

    color += Filter22(text, uv, scale, lod) * 0.5;
    for(int i = 0; i < 4; i++){
        vec2 offset = FilterOffset5_22[i] * deltaRes;
        vec2 newUV = uv + offset;
        color += Filter22(text, newUV, scale, lod) * 0.125;
    }

    return color;
}



// 3*3 帐篷滤波
vec3 tentFilter33(sampler2D text, vec2 uv, float lod, float scale){
    int k = 0;
    vec3 sampleTex[9];

    int r = int(scale);
    for(int i = -r; i <= r; i += r) {
    for(int j = -r; j <= r; j += r) {
        vec2 offset = vec2(float(i)/viewWidth, float(j)/viewHeight);
        vec2 newUV = uv + offset;

        // 剔除屏幕外采样
        // vec2 testUV = (newUV - vec2(uvOffset, 0.0))/uvScale;
        // if(outScreen(testUV, (1.0 / uvScale)/2.0 + 0.1)){
        //     sampleTex[k] = texture(text, uv).rgb;
        // }else{
            sampleTex[k] = textureLod(text, newUV, lod).rgb;
        // }
        // newUV = testUV * uvScale + vec2(uvOffset, 0.0);      
        k++;
    }
    }

    vec3 color = vec3(0.0);

    const float weights[9] = float[](
        1.0/16.0, 2.0/16.0, 1.0/16.0,
        2.0/16.0, 4.0/16.0, 2.0/16.0,
        1.0/16.0, 2.0/16.0, 1.0/16.0
    );
    for(int i = 0; i < 9; i++) {
        color += sampleTex[i] * weights[i];
    }

    return color;
}



// 5*5 高斯模糊
const float weights[25] = float[](
    0.003765, 0.015019, 0.023792, 0.015019, 0.003765,
    0.015019, 0.059912, 0.094907, 0.059912, 0.015019,
    0.023792, 0.094907, 0.150342, 0.094907, 0.023792,
    0.015019, 0.059912, 0.094907, 0.059912, 0.015019,
    0.003765, 0.015019, 0.023792, 0.015019, 0.003765);

vec3 gaussianBlur55(sampler2D text, vec2 uv, float scale, float lod){
    // if(outScreen(uv, scale / 2.0 + 0.1)) 
    //     return vec3(0.0);
    
    vec2 deltaRes = vec2(1.0 / viewWidth, 1.0 / viewHeight);
    vec3 result = vec3(0.0);
    float weightSum = 0.0;

    for (int i = -2; i <= 2; i++) {
    for (int j = -2; j <= 2; j++) {
        vec2 offset = vec2(float(i), float(j)) * scale * deltaRes;
        result += textureLod(text, uv + offset, lod).rgb * weights[(i + 2) * 5 + (j + 2)];
        weightSum += weights[(i + 2) * 5 + (j + 2)];
    }
    }
    return result / weightSum;
}

// 纵横向高斯模糊
float gaussianWeights[5] = {0.06136, 0.24477, 0.38774, 0.24477, 0.06136};
// float gaussianWeights[5] = {0.2, 0.2, 0.2, 0.2, 0.2};

void calculateGaussianWeights(float weights[5], float sigma) {
    float sum = 0.0;
    for (int i = -2; i <= 2; i++) {
        float x = float(i);
        weights[i + 2] = exp(-(x * x) / (2.0 * sigma * sigma));
        sum += weights[i + 2];
    }
    // 归一化权重
    for (int i = 0; i < 5; i++) {
        weights[i] /= sum;
    }
}

vec3 horizontalGaussianBlur(sampler2D text, vec2 uv, float scale, float lod) {
    // if(outScreen(uv, scale / 2.0 + 1.0)) 
    //     return vec3(0.0);

    vec2 texOffset = vec2(1.0) / texResolution;
    vec3 result = vec3(0.0);
    for (int i = -2; i <= 2; i++) {
        vec2 offset = vec2(float(i), 0.0) * texOffset * scale;
        result += textureLod(text, uv + offset, lod).rgb * gaussianWeights[i + 2];
    }
    return result;
}

vec3 verticalGaussianBlur(sampler2D text, vec2 uv, float scale, float lod) {
    // if(outScreen(uv, scale / 2.0 + 1.0)) 
    //     return vec3(0.0);

    vec2 texOffset = vec2(1.0) / texResolution;
    vec3 result = vec3(0.0);
    for (int i = -2; i <= 2; i++) {
        vec2 offset = vec2(0.0, float(i)) * texOffset * scale;
        result += textureLod(text, uv + offset, lod).rgb * gaussianWeights[i + 2];
    }
    return result;
}

// existical : Gaussian Blur Simple and Fast 
// https://www.shadertoy.com/view/Xltfzj
vec3 GaussianBlurSimple(sampler2D text, vec2 uv, float Directions, float Quality, float Size){
    // GAUSSIAN BLUR SETTINGS
    // BLUR DIRECTIONS (Default 16.0 - More is better but slower)
    // BLUR QUALITY (Default 4.0 - More is better but slower)
    // BLUR SIZE (Radius)

    vec2 Radius = Size/texResolution;

    vec3 Color = texture(text, uv).rgb;
    // Blur calculations
    for( float d=0.0; d < _2PI; d += _2PI / Directions){
		for(float i=1.0/Quality; i<=1.0; i+=1.0/Quality){
			Color += texture( text, uv+vec2(cos(d),sin(d))*Radius*i).rgb;		
        }
    }
    Color /= Quality * Directions;
    return Color; 
}



// simesgreen : https://www.shadertoy.com/view/4df3Dn
// 4*4 bicubic filter
// Catmull-Rom bicubic
float w0(float a){
    return (1.0/6.0)*(a*(a*(-a + 3.0) - 3.0) + 1.0);
}

float w1(float a){
    return (1.0/6.0)*(a*a*(3.0*a - 6.0) + 4.0);
}

float w2(float a){
    return (1.0/6.0)*(a*(a*(-3.0*a + 3.0) + 3.0) + 1.0);
}

float w3(float a){
    return (1.0/6.0)*(a*a*a);
}

// g0 and g1 are the two amplitude functions
float g0(float a){
    return w0(a) + w1(a);
}

float g1(float a){
    return w2(a) + w3(a);
}

// h0 and h1 are the two offset functions
float h0(float a){
    return -1.0 + w1(a) / (w0(a) + w1(a));
}

float h1(float a){
    return 1.0 + w3(a) / (w2(a) + w3(a));
}

vec4 textureBicubic(sampler2D text, vec2 uv){
	uv = uv * texResolution + 0.5;
	vec2 iuv = floor( uv );
	vec2 fuv = fract( uv );

    float g0x = g0(fuv.x);
    float g1x = g1(fuv.x);
    float h0x = h0(fuv.x);
    float h1x = h1(fuv.x);
    float h0y = h0(fuv.y);
    float h1y = h1(fuv.y);

	vec2 p0 = (vec2(iuv.x + h0x, iuv.y + h0y) - 0.5) / texResolution;
	vec2 p1 = (vec2(iuv.x + h1x, iuv.y + h0y) - 0.5) / texResolution;
	vec2 p2 = (vec2(iuv.x + h0x, iuv.y + h1y) - 0.5) / texResolution;
	vec2 p3 = (vec2(iuv.x + h1x, iuv.y + h1y) - 0.5) / texResolution;
	
    return g0(fuv.y) * (g0x * texture(text, p0)  +
                        g1x * texture(text, p1)) +
           g1(fuv.y) * (g0x * texture(text, p2)  +
                        g1x * texture(text, p3));
}





// hornet: Texture Filtering: Catmull-Rom 
// https://www.shadertoy.com/view/MtVGWz
vec3 catmullRom(sampler2D text, vec2 uv){
    vec2 samplePos = uv * texResolution;
    vec2 texPos1 = floor(samplePos - 0.5f) + 0.5f;
    vec2 f = samplePos - texPos1;
    vec2 w0 = f * (-0.5f + f * (1.0f - 0.5f * f));
    vec2 w1 = 1.0f + f * f * (-2.5f + 1.5f * f);
    vec2 w2 = f * (0.5f + f * (2.0f - 1.5f * f));
    vec2 w3 = f * f * (-0.5f + 0.5f * f);
    vec2 w12 = w1 + w2;
    vec2 offset12 = w2 / (w1 + w2);
    vec2 texPos0 = texPos1 - 1.0f;
    vec2 texPos3 = texPos1 + 2.0f;
    vec2 texPos12 = texPos1 + offset12;
    texPos0 /= texResolution;
    texPos3 /= texResolution;
    texPos12 /= texResolution;
    vec3 result = vec3(0.0f, 0.0f, 0.0f);
    result += texture(text, vec2(texPos0.x, texPos0.y)).xyz * w0.x * w0.y;
    result += texture(text, vec2(texPos12.x, texPos0.y)).xyz * w12.x * w0.y;
    result += texture(text, vec2(texPos3.x, texPos0.y)).xyz * w3.x * w0.y;
    result += texture(text, vec2(texPos0.x, texPos12.y)).xyz * w0.x * w12.y;
    result += texture(text, vec2(texPos12.x, texPos12.y)).xyz * w12.x * w12.y;
    result += texture(text, vec2(texPos3.x, texPos12.y)).xyz * w3.x * w12.y;
    result += texture(text, vec2(texPos0.x, texPos3.y)).xyz * w0.x * w3.y;
    result += texture(text, vec2(texPos12.x, texPos3.y)).xyz * w12.x * w3.y;
    result += texture(text, vec2(texPos3.x, texPos3.y)).xyz * w3.x * w3.y;
    return max(result, 0.0f);
}

vec4 catmullRom5( sampler2D text, vec2 uv, float sharpenAmount){
    vec2 texsiz = texResolution;
    vec4 rtMetrics = vec4( 1.0 / texsiz.xy, texsiz.xy );
    
    vec2 position = rtMetrics.zw * uv;
    vec2 centerPosition = floor(position - 0.5) + 0.5;
    vec2 f = position - centerPosition;
    vec2 f2 = f * f;
    vec2 f3 = f * f2;

    const float c = sharpenAmount;
    vec2 w0 =        -c  * f3 +  2.0 * c         * f2 - c * f;
    vec2 w1 =  (2.0 - c) * f3 - (3.0 - c)        * f2         + 1.0;
    vec2 w2 = -(2.0 - c) * f3 + (3.0 -  2.0 * c) * f2 + c * f;
    vec2 w3 =         c  * f3 -                c * f2;

    vec2 w12 = w1 + w2;
    vec2 tc12 = rtMetrics.xy * (centerPosition + w2 / w12);
    vec3 centerColor = texture(text, vec2(tc12.x, tc12.y)).rgb;

    vec2 tc0 = rtMetrics.xy * (centerPosition - 1.0);
    vec2 tc3 = rtMetrics.xy * (centerPosition + 2.0);
    vec4 color = vec4(texture(text, vec2(tc12.x, tc0.y )).rgb, 1.0) * (w12.x * w0.y ) +
                 vec4(texture(text, vec2(tc0.x,  tc12.y)).rgb, 1.0) * (w0.x  * w12.y) +
                 vec4(centerColor,                                 1.0) * (w12.x * w12.y) +
                 vec4(texture(text, vec2(tc3.x,  tc12.y)).rgb, 1.0) * (w3.x  * w12.y) +
                 vec4(texture(text, vec2(tc12.x, tc3.y )).rgb, 1.0) * (w12.x * w3.y );
    return vec4( color.rgb / color.a, 1.0 );
}



// void poissonDiskSamples(const in vec2 randomSeed, const float quality) {
//     float angle = rand2_1(randomSeed) * _2PI;
//     float inv_q = 1.0 / quality;
//     float radius = inv_q;

//     float ANGLE_STEP = 3.883222077450933; // (sqrt(5)-1)/2 * 2PI
//     float radiusStep = radius;

//     vec2 disk[quality];

//     for (int i = 0; i < quality; i++) {
//         disk[i] = vec2(cos(angle), sin(angle)) * pow(radius, 0.75);
//         radius += radiusStep;
//         angle += ANGLE_STEP;
//     }
// }



// iq :Texture - Better Filtering 
// https://www.shadertoy.com/view/XsfGDn
vec3 textureNice( sampler2D sam, vec2 uv ){
    uv = uv*texResolution + 0.5;
    vec2 iuv = floor( uv );
    vec2 fuv = fract( uv );
    uv = iuv + fuv*fuv*(3.0-2.0*fuv);
    uv = (uv - 0.5)/texResolution;
    return texture( sam, uv ).rgb;
}

vec3 textureNice( sampler2D sam, vec2 uv, vec2 texRes ){
    uv = uv*texRes + 0.5;
    vec2 iuv = floor( uv );
    vec2 fuv = fract( uv );
    uv = iuv + fuv*fuv*(3.0-2.0*fuv);
    uv = (uv - 0.5)/texRes;
    return texture( sam, uv ).rgb;
}

vec3 textureNice( sampler2D sam, vec2 uv, vec2 texRes, float lod ){
    uv = uv*texRes + 0.5;
    vec2 iuv = floor( uv );
    vec2 fuv = fract( uv );
    uv = iuv + fuv*fuv*(3.0-2.0*fuv);
    uv = (uv - 0.5)/texRes;
    return textureLod( sam, uv, lod ).rgb;
}




float triangleWave(float x) {
    return abs(fract(x + 0.5) * 2.0 - 1.0);
}

float smoothCurve(float x){
    return (3.0 - 2.0 * x) * x * x;
}



// 高斯函数
float gaussian(float x, float sigma) {
    return exp(-0.5 * (x * x) / (sigma * sigma));
}

// 双边滤波函数
vec3 bilateralFilter(sampler2D text, vec2 uv, float radius, float scale) {
    float sigmaS = radius / 2.0;
    float sigmaR = scale;
    vec3 centerColor = texture(text, uv).rgb;

    float totalWeight = 0.0;
    vec3 resultColor = vec3(0.0);

    // 遍历邻域像素
    for (float x = -radius; x <= radius; ++x) {
        for (float y = -radius; y <= radius; ++y) {
            vec2 offset = vec2(x, y) / texResolution;
            vec3 sampleColor = texture(text, uv + offset).rgb;

            // 空间权重
            float spatialWeight = gaussian(length(vec2(x, y)), sigmaS);
            // 强度权重
            float rangeWeight = gaussian(length(sampleColor - centerColor), sigmaR);

            float weight = spatialWeight * rangeWeight;
            totalWeight += weight;
            resultColor += sampleColor * weight;
        }
    }

    return resultColor / totalWeight;
}



