// 数学
float checkInRange(float value, float targetValue, float range) {
    if ((value > targetValue - range) && value < (targetValue + range)) {
        return 1.0;
    } else {
        return 0.0;
    }
}

//clamp 0到1
float clamp01(float x) {
	return clamp(x, 0.0, 1.0);
}

vec2 clamp01(vec2 x) {
	return clamp(x, vec2(0.0), vec2(1.0));
}

vec3 clamp01(vec3 x) {
	return clamp(x, vec3(0.0), vec3(1.0));
}

// 旋转矩阵
mat2 rotate2D(float angle) {
    float s = sin(angle);
    float c = cos(angle);
    
    return mat2(
        c, -s,
        s,  c
    );
}

mat3 rotate3D(vec3 axis, float angle) {
    axis = normalize(axis);  // 确保轴是单位向量
    float s = sin(angle);
    float c = cos(angle);
    float oc = 1.0 - c;
    
    return mat3(
        oc * axis.x * axis.x + c,           oc * axis.x * axis.y - axis.z * s,  oc * axis.x * axis.z + axis.y * s,
        oc * axis.x * axis.y + axis.z * s,  oc * axis.y * axis.y + c,           oc * axis.y * axis.z - axis.x * s,
        oc * axis.x * axis.z - axis.y * s,  oc * axis.y * axis.z + axis.x * s,  oc * axis.z * axis.z + c
    );
}



// 球面求交
vec2 RaySphereIntersection(vec3 rayOrigin, vec3 rayDir, vec3 sphereCenter, float sphereRadius) {
    rayOrigin -= sphereCenter;
    float a = dot(rayDir, rayDir);
    float b = 2.0 * dot(rayOrigin, rayDir);
    float c = dot(rayOrigin, rayOrigin) - (sphereRadius * sphereRadius);
    float d = b * b - 4.0 * a * c;

    if (d < 0.0) {
        return vec2(-1.0);
    } else {
        d = sqrt(d);
        return vec2(-b - d, -b + d) / (2.0 * a);
    }
}

// 包围盒求交
vec2 intersectAABB_Y(in vec3 origin, in vec3 direction, float boxHeight, float boxThickness) {
    // Calculate the y boundaries of the box
    float halfThickness = boxThickness / 2.0;
    float boxMinY = boxHeight - halfThickness;
    float boxMaxY = boxHeight + halfThickness;
    
    // Calculate intersection distances with the y planes of the box
    float t1 = (boxMinY - origin.y) / direction.y;
    float t2 = (boxMaxY - origin.y) / direction.y;

    // Ensure t1 is the smaller value
    if (t1 > t2) {
        float temp = t1;
        t1 = t2;
        t2 = temp;
    }

    return vec2(t1, t2);
}



// 颜色混合
float lerp(float a, float b, float f){
    return a + f * (b - a);
}

vec3 lerp(vec3 a, vec3 b, float f){
    return a + f * (b - a);
}



// 数据操作
float pack2x8Bits(vec2 data8){
    float red = floor(data8.x * 255.0 + 0.5);
    float green = floor(data8.y * 255.0 + 0.5);
    
    float Packed = red * 256.0 + green;
    
    return Packed / 65535.0;
}

vec2 unpack16Bit(float data16){
    float Packed = data16 * 65535.0;
    
    float red = floor(Packed / 256.0);
    float green = Packed - red * 256.0;

    float redNormalized = red / 255.0;
    float greenNormalized = green / 255.0;
    
    return vec2(redNormalized, greenNormalized);
}

float packRGBToFloat(vec3 rgb) {
    vec3 scaledRGB = floor(rgb * 255.0 + 0.5);

    int r = int(scaledRGB.r);
    int g = int(scaledRGB.g);
    int b = int(scaledRGB.b);

    int packedInt = (r << 16) | (g << 8) | b;

    return float(packedInt);
}

vec3 unpackFloatToRGB(float packedFloat) {
    int packedInt = int(packedFloat);

    int r = (packedInt >> 16) & 0xFF;
    int g = (packedInt >> 8) & 0xFF;
    int b = packedInt & 0xFF;

    vec3 rgb = vec3(float(r), float(g), float(b)) / 255.0;

    return rgb;
}