// Loka: 【shader】超级噪声库，附代码（fbm、Perlin、Simplex、Worley、Tiling、Curl等，很全很全）
// https://zhuanlan.zhihu.com/p/560229938

float rand3_1(vec3 co) {
    float a = 12.9898;
    float b = 78421.24892;
    float c = 43758.54538;
    float dt= dot(co.xyz, vec3(a, b, c));
    float sn= mod(dt, 3.14);
    return fract(sin(sn) * c);
}

float rand2_1(vec2 p) {
    vec2 a = vec2(12.9898, 78.233);
    float b = 0.9753;
    float dt = dot(p, a);
    float sine = sin(dt) * 43758.5453 + b;
    return fract(sin(sine) * 43758.5453123);
}

vec3 rand2_3(vec2 p) {
    vec2 a = vec2(12.9898, 78.233);
    vec3 b = vec3(0.543, 0.2795, 0.9753);
    vec3 dt = vec3(dot(p, a),
                dot(p, a * 2.0),
                dot(p, a * 3.0));
    vec3 sine = sin(dt) * 43758.5453 + b;
    return fract(sin(sine) * 43758.5453123);
}

vec3 rand3_3(vec3 p) {
    vec3 a = vec3(12.9898, 78.233, 45.164);
    vec3 b = vec3(0.543, 0.2795, 0.9753);
    vec3 dt = vec3(dot(p, a),
                dot(p, a * 2.0),
                dot(p, a * 3.0));
    vec3 sine = sin(dt) * 43758.5453 + b;
    return fract(sin(sine) * 43758.5453123);
}



float BlueNoise(vec2 coord, float scale){
	vec2 noiseCoord = vec2(coord.st * texResolution) / scale;
	noiseCoord += vec2(sin(frameCounter * 0.75), cos(frameCounter * 0.75));
	noiseCoord = (floor(noiseCoord * scale) + 0.5) / scale;

	float blueNoise = textureLod(noisetex, noiseCoord.st, 0).b;
	return blueNoise;
}



float r(float n){
 	return fract(cos(n*89.42)*343.42);
}
vec2 r(vec2 n){
 	return vec2(r(n.x*23.62-300.0+n.y*34.35),r(n.x*45.13+256.0+n.y*38.89)); 
}
float worley(vec2 n,float s){
    float dis = 2.0;
    for(int x = -1;x<=1;x++)
    {
        for(int y = -1;y<=1;y++)
        {
            vec2 p = floor(n/s)+vec2(x,y);
            float d = length(r(p)+vec2(x,y)-fract(n/s));
            if (dis>d)
            {
             	dis = d;   
            }
        }
    }
    return 1.0 - dis;
}

// piyushslaye: Tileable Perlin-Worley 3D 
// https://www.shadertoy.com/view/3dVXDc
#define UI0 1597334673U
#define UI1 3812015801U
#define UI2 uvec2(UI0, UI1)
#define UI3 uvec3(UI0, UI1, 2798796415U)
#define UIF (1.0 / float(0xffffffffU))

vec3 hash33w(vec3 p){
	uvec3 q = uvec3(ivec3(p)) * UI3;
	q = (q.x ^ q.y ^ q.z)*UI3;
	return -1. + 2. * vec3(q) * UIF;
}

float worley3D(vec3 uv, float freq){    
    vec3 id = floor(uv);
    vec3 p = fract(uv);
    
    float minDist = 10000.;
    for (float x = -1.; x <= 1.; ++x)
    {
        for(float y = -1.; y <= 1.; ++y)
        {
            for(float z = -1.; z <= 1.; ++z)
            {
                vec3 offset = vec3(x, y, z);
            	vec3 h = hash33w(mod(id + offset, vec3(freq))) * .5 + .5;
    			h += offset;
            	vec3 d = p - h;
           		minDist = min(minDist, dot(d, d));
            }
        }
    }
    
    // inverted worley noise
    return 1. - minDist;
}



#define MOD3 vec3(.1031,.11369,.13787)

vec3 hash33(vec3 p3){
	p3 = fract(p3 * MOD3);
    p3 += dot(p3, p3.yxz+19.19);
    return -1.0 + 2.0 * fract(vec3((p3.x + p3.y)*p3.z, (p3.x+p3.z)*p3.y, (p3.y+p3.z)*p3.x));
}

float PerLin3d(vec3 p){
    vec3 pi = floor(p);
    vec3 pf = p - pi;
    
    vec3 w = pf * pf * (3.0 - 2.0 * pf);
    
    return 	mix(
        		mix(
                	mix(dot(pf - vec3(0, 0, 0), hash33(pi + vec3(0, 0, 0))), 
                        dot(pf - vec3(1, 0, 0), hash33(pi + vec3(1, 0, 0))),
                       	w.x),
                	mix(dot(pf - vec3(0, 0, 1), hash33(pi + vec3(0, 0, 1))), 
                        dot(pf - vec3(1, 0, 1), hash33(pi + vec3(1, 0, 1))),
                       	w.x),
                	w.z),
        		mix(
                    mix(dot(pf - vec3(0, 1, 0), hash33(pi + vec3(0, 1, 0))), 
                        dot(pf - vec3(1, 1, 0), hash33(pi + vec3(1, 1, 0))),
                       	w.x),
                   	mix(dot(pf - vec3(0, 1, 1), hash33(pi + vec3(0, 1, 1))), 
                        dot(pf - vec3(1, 1, 1), hash33(pi + vec3(1, 1, 1))),
                       	w.x),
                	w.z),
    			w.y);
}



 /* discontinuous pseudorandom uniformly distributed in [-0.5, +0.5]^3 */
vec3 random3(vec3 c) {
    float j = 4096.0*sin(dot(c,vec3(17.0, 59.4, 15.0)));
    vec3 r;
    r.z = fract(512.0*j);
    j *= .125;
    r.x = fract(512.0*j);
    j *= .125;
    r.y = fract(512.0*j);
    return r-0.5;
}

/* skew constants for 3d simplex functions */
const float F3 =  0.3333333;
const float G3 =  0.1666667;

/* 3d simplex noise */
float simplex3d(vec3 p) {
    /* 1. find current tetrahedron T and it's four vertices */
    /* s, s+i1, s+i2, s+1.0 - absolute skewed (integer) coordinates of T vertices */
    /* x, x1, x2, x3 - unskewed coordinates of p relative to each of T vertices*/
    
    /* calculate s and x */
    vec3 s = floor(p + dot(p, vec3(F3, F3, F3)));
    vec3 x = p - s + dot(s, vec3(G3, G3, G3));
    
    /* calculate i1 and i2 */
    vec3 e = step(vec3(0,0,0), x - x.yzx);
    vec3 i1 = e*(1.0 - e.zxy);
    vec3 i2 = 1.0 - e.zxy*(1.0 - e);
    
    /* x1, x2, x3 */
    vec3 x1 = x - i1 + G3;
    vec3 x2 = x - i2 + 2.0*G3;
    vec3 x3 = x - 1.0 + 3.0*G3;
    
    /* 2. find four surflets and store them in d */
    vec4 w, d;
    
    /* calculate surflet weights */
    w.x = dot(x, x);
    w.y = dot(x1, x1);
    w.z = dot(x2, x2);
    w.w = dot(x3, x3);
    
    /* w fades from 0.6 at the center of the surflet to 0.0 at the margin */
    w = max(0.6 - w, 0.0);
    
    /* calculate surflet components */
    d.x = dot(random3(s), x);
    d.y = dot(random3(s + i1), x1);
    d.z = dot(random3(s + i2), x2);
    d.w = dot(random3(s + 1.0), x3);
    
    /* multiply d by w^4 */
    w *= w;
    w *= w;
    d *= w;
    
    /* 3. return the sum of the four surflets */
    return dot(d, vec4(52.0, 52.0, 52.0, 52.0));
}



vec3 textureN( sampler2D sam, vec2 uv ){
    uv = uv*noiseTextureResolution + 0.5;
    vec2 iuv = floor( uv );
    vec2 fuv = fract( uv );
    uv = iuv + fuv*fuv*(3.0-2.0*fuv);
    uv = (uv - 0.5)/noiseTextureResolution;
    return texture( sam, uv ).rgb;
}

// 参考自 SEUS
float noisetex2_3(vec3 pos){
	pos.xyz += 0.5f;

	vec3 p = floor(pos);
	vec3 f = fract(pos);

	vec2 uv =  (p.xy + p.z * vec2(17.0f)) + f.xy;
	vec2 uv2 = (p.xy + (p.z + 1.0f) * vec2(17.0f)) + f.xy;

	vec2 coord =  (uv  + 0.5f) / noiseTextureResolution;
	vec2 coord2 = (uv2 + 0.5f) / noiseTextureResolution;
	float xy1 = textureN(noisetex, coord).x;
	float xy2 = textureN(noisetex, coord2).x;
	float n = mix(xy1, xy2, f.z);
    return n * 2.0 - 1.0;
}



// from iq
#define NOISE_METHOD 1
float noisetex2_3A(vec3 x){
    vec3 p = floor(x);
    vec3 f = fract(x);
	f = f*f*(3.0-2.0*f);

#if NOISE_METHOD==0
    x = p + f;
    return textureLod(noisetex,(x+0.5)/32.0,0.0).x*2.0-1.0;
#endif
#if NOISE_METHOD==1
	vec2 uv = (p.xy+vec2(37.0,239.0)*p.z) + f.xy;
    vec2 rg = textureLod(noisetex,(uv+0.5)/256.0,0.0).yx;
	return mix( rg.x, rg.y, f.z )*2.0-1.0;
#endif    
#if NOISE_METHOD==2
    ivec3 q = ivec3(p);
	ivec2 uv = q.xy + ivec2(37,239)*q.z;
	vec2 rg = mix(mix(texelFetch(noisetex,(uv           )&255,0),
				      texelFetch(noisetex,(uv+ivec2(1,0))&255,0),f.x),
				  mix(texelFetch(noisetex,(uv+ivec2(0,1))&255,0),
				      texelFetch(noisetex,(uv+ivec2(1,1))&255,0),f.x),f.y).yx;
	return mix( rg.x, rg.y, f.z );
#endif   
}