vec4 viewPosToScreenPos(vec4 viewPos){
    vec4 clipPos = gbufferProjection * viewPos;
    vec3 NDCPos = clipPos.xyz / clipPos.w;
    
    return vec4(NDCPos.xyz*0.5 + 0.5, 1.0);
}

vec4 screenPosToViewPos(vec4 screenPos){
    vec4 NDCPos = vec4(screenPos.xyz * 2.0 - 1.0, 1.0);
    vec4 clipPos = gbufferProjectionInverse * NDCPos;
    return vec4(clipPos.xyz / clipPos.w, 1.0);
}

vec4 viewPosToWorldPos(vec4 viewPos){
    return gbufferModelViewInverse * viewPos;
}

vec4 getShadowPos(vec4 worldPos){
    vec4 sClipPos = shadowMat * worldPos;
    vec4 sNDCPos = vec4(sClipPos.xyz / sClipPos.w, 1.0);
        float sDist = length(sNDCPos.xy);
        float distortFactor = (1.0 - shadowMap_BIAS) + sDist * shadowMap_BIAS;
        sNDCPos.xy /= distortFactor;
        sNDCPos.z = mix(sNDCPos.z, 0.5, 0.8);
    vec4 sScreenPos = sNDCPos * 0.5 + 0.5;

    return sScreenPos;
}

vec4 getShadowViewPos(vec4 shadowPos, float lod){
    float dist = textureLod(shadowcolor1, shadowPos.xy, lod).a * 1.41421356237;
    shadowPos = vec4(shadowPos.xyz * 2.0 - 1.0, 1.0);
    shadowPos.z = (shadowPos.z - 0.5 * 0.8) / 0.2;
    shadowPos.xy *= (1.0 - shadowMap_BIAS) + dist * shadowMap_BIAS;
    shadowPos = shadowProjectionInverse * shadowPos;

    return vec4(shadowPos.xyz / shadowPos.w, 1.0);
}



float linearDepthToExpDepth(float depth){
    float z_ndc = (2.0 * near * far) / (far + near - depth * (far - near));
    return (z_ndc + 1.0) / 2.0; // 将 z_ndc 转换到 [0, 1] 范围
}

float expDepthToLinearDepth(float depth){
    float z = depth * 2.0 - 1.0; // 将 depth 转换到 [-1, 1] 的范围
    return (2.0 * near * far) / (far + near - z * (far - near));
}

float linearDepth(vec2 uv) {					
	return (near * far) / (texture(depthtex1, uv).r * (near - far) + far);
}



bool outScreen(vec2 uv, float offset){
    if(uv.x <= 0.0 + offset / viewWidth 
    || uv.x >= 1.0 - offset / viewWidth 
    || uv.y <= 0.0 + offset / viewHeight 
    || uv.y >= 1.0 - offset / viewHeight){
        return true;
    }else{
        return false;
    }
}

bool outScreen(vec2 uv){
    if(uv.x <= 0.0
    || uv.x >= 1.0
    || uv.y <= 0.0
    || uv.y >= 1.0){
        return true;
    }else{
        return false;
    }
}

bool outScreen(vec3 uv){
    if((uv.x <= 0.0
    || uv.x >= 1.0
    || uv.y <= 0.0
    || uv.y >= 1.0)
    && uv.z <= 0.0){
        return true;
    }else{
        return false;
    }
}



//筛选出点附近最小深度（最接近摄像机）的位置
vec3 getClosestOffset(vec2 uv, float scale){
    vec2 deltaRes = vec2(1.0 / viewWidth, 1.0 / viewHeight);
    float closestDepth = 1.0f;
    vec2 closestUV = uv;

    for(int i = -1; i <= 1; i++){
    for(int j = -1; j <= 1; j++){
        vec2 newUV = uv + scale * deltaRes * vec2(i, j);
        float depth = texture(depthtex1, newUV).r;

        if(depth < closestDepth){
            closestDepth = depth;
            closestUV = newUV;
        }
    }
    }
    return vec3(closestUV, closestDepth);
}