// WC Yang: 高质量实时渲染：实时全局光照
// https://yangwc.com/2021/06/12/SSR/

//感谢 Tahnass 在实现过程中提供的帮助

vec2 poissonDisk[16] = vec2[](
    vec2( -0.94201624, -0.39906216 ),
    vec2( 0.94558609, -0.76890725 ),
    vec2( -0.094184101, -0.92938870 ),
    vec2( 0.34495938, 0.29387760 ),
    vec2( -0.91588581, 0.45771432 ),
    vec2( -0.81544232, -0.87912464 ),
    vec2( -0.38277543, 0.27676845 ),
    vec2( 0.97484398, 0.75648379 ),
    vec2( 0.44323325, -0.97511554 ),
    vec2( 0.53742981, -0.47373420 ),
    vec2( -0.26496911, -0.41893023 ),
    vec2( 0.79197514, 0.19090188 ),
    vec2( -0.24188840, 0.99706507 ),
    vec2( -0.81409955, 0.91437590 ),
    vec2( 0.19984126, 0.78641367 ),
    vec2( 0.14383161, -0.14100790 )
);

vec3 	CalculateNoisePattern(vec2 offset, float size) {
	vec2 coord = texcoord.st;

	coord *= vec2(viewWidth, viewHeight);
	coord = mod(coord + offset, vec2(size));
	coord /= noiseTextureResolution;

	return texture2D(noisetex, coord).xyz;
}

vec3 RSM(vec3 normalW, vec4 worldPos){
    vec3 normalSP = normalize((shadowModelView * vec4(normalW, 0.0)).xyz);

    vec4 sViewPos = shadowModelView * worldPos;
    vec4 sClipPos = shadowProjection * sViewPos;
    vec4 sNDCPos = vec4(sClipPos.xyz / sClipPos.w, 1.0);
        float sDist = length(sNDCPos.xy);
        float distortFactor = (1.0 - shadowMap_BIAS) + sDist * shadowMap_BIAS;
        sNDCPos.xy /= distortFactor;
        sNDCPos.z = mix(sNDCPos.z, 0.5, 0.8);
    vec4 sScreenPos = sNDCPos * 0.5 + 0.5;

    float c = 0;
    vec3 rsmColor = vec3(0.0);

    float distFactor = length(worldPos.xyz);
    distFactor = clamp(distFactor / 20, 1.0, 3.0);

#ifdef RSM_M1
    const float radius = 80.0;
    const float quality = 3.0 / distFactor;
    float d = 2.0 * radius / quality;
    float lod = log2(d);
    float noise = rand2_1(texcoord + 1.0 * sin(1.0 * frameTimeCounter));

    for(float i = -radius; i < radius + 0.1; i += d){
    for(float j = -radius; j < radius + 0.1; j += d){
        // if(abs(i) < 0.1 || abs(j) < 0.1) break;
        // lod += 0.5 * (abs(i) + abs(j)) / radius;
        vec2 offset = vec2(i, j);
        vec2 noiseXZ = CalculateNoisePattern(vec2(0.0), 2).rb;
        offset += vec2(noiseXZ - 0.5) * d;  
        // offset = offset * rotate2D(noise * _2PI);
        offset /= shadowMapResolution;
        vec2 nowPos = sScreenPos.xy + offset;
        float nowDepth = textureLod(shadowtex0, nowPos, lod).r;
        vec3 nowSVPos = getShadowViewPos(vec4(nowPos, nowDepth, 1.0), lod).xyz;

        vec3 vector = nowSVPos - sViewPos.xyz;
        vec3 dir = normalize(vector);
        float dist = length(vector);
        dist = dist * dist;

        vec3 nowVNormal = normalize(textureLod(shadowcolor1, nowPos, lod).xyz * 2.0 - 1.0);
        vec3 nowColor = gammaToLinearR(textureLod(shadowcolor0, nowPos, lod + 0.1 * dist).rgb);
        float normalFactor = max(0.0, dot(normalSP, dir)) * max(0.0, dot(nowVNormal, -dir));
        rsmColor += nowColor * normalFactor / (dist + 5);
        ++c;
    }
    }
#endif

#ifdef RSM_M2
    // 每次循环跨越的角度和距离
    float ANGLE_STEP = 3.883222077450933; // (sqrt(5)-1)/2 * 2PI
    float radiusStep = 40.0 * distFactor;

    // 设置起点（初始角度，距离），应用noise
    float noise = rand2_1(texcoord + 1.0 * sin(1.0 * frameTimeCounter));
    // noise = 0.0;
    float angle = noise * _2PI;
    float radius = noise * radiusStep * 2.0;

    const float quality = 15.0;
    for (int i = 0; i < quality / distFactor; i++) {
        float lod = max(log2(i * 16.0 / quality), 1.0);
        // lod = 6.0;

        vec2 offset = vec2(cos(angle), sin(angle)) * pow(radius, 0.75);
        // vec2 offset = poissonDisk[i] * 80;
        // offset += vec2(noise.xx - 0.5) * 1; 
        // offset = rotate2D(angle) * offset;

        offset /= shadowMapResolution;
        vec3 nowPos = vec3(sScreenPos.xy + offset, 0.0);
        nowPos.z = textureLod(shadowtex0, nowPos.st, lod).r;
        vec3 nowSViewPos = getShadowViewPos(vec4(nowPos, 1.0), lod).xyz;

        vec3 vector = nowSViewPos - sViewPos.xyz;
        vec3 dir = normalize(vector);
        float dist = length(vector);

        vec3 nowNormal = normalize(textureLod(shadowcolor1, nowPos.st, lod).xyz * 2.0 - 1.0);
        float pq_dot_pn = max(0.0, dot(normalSP, dir));
        float qp_dot_qn = max(0.0, dot(nowNormal, -dir));
        dist = dist * dist;

        vec3 flux = gammaToLinearR(textureLod(shadowcolor0, nowPos.st, lod + 0.35 * dist).rgb);

        float weight = max(lod, 1.0);
        // weight = 1.0;
        rsmColor += weight * flux * pq_dot_pn * qp_dot_qn / (dist + 1.0);
        c += weight;
        // c += 1.0;

        radius += radiusStep;
        angle += ANGLE_STEP;
    }
#endif

    return max(BLACK, rsmColor / max(c, 0.01));
}