float HBAO(vec2 uv, float z, vec3 normal){
    vec4 viewPos = screenPosToViewPos(vec4(uv, z, 1.0));
    float dis = max(length(viewPos.xyz), 1.0);

    float noise = rand2_1(uv + sin(frameTimeCounter));

    const float dirCount = 10.0;
    float dDir = _2PI / dirCount;
    float firstDir = noise * dDir;

    const float stepCount = 10.0;
    const float radius = clamp01(1.5 / dis);
    float dStep = radius / stepCount;

    float ao = 0.0;
    for(float dir = firstDir; dir < _2PI; dir += dDir){
        float maxAO = 0.0;
        for(int i = 1; i < stepCount + 0.1; ++i){
            vec2 nowUV = uv + vec2(cos(dir), sin(dir)) * dStep * (i - noise);
            float nowZ = texture(depthtex1, nowUV).r;
            vec3 nowViewPos = screenPosToViewPos(vec4(nowUV, nowZ, 1.0)).xyz;
            vec3 H = normalize(nowViewPos - viewPos.xyz);

        float nowAO = 0.0;
        #define HBAO_M2
        #ifdef HBAO_M1
            vec3 T = vec3(H.xy, 0.0);
            T.z = -(T.x * normal.x + T.y * normal.y) / normal.z;
            T = normalize(T);

            float h = atan(H.z / length(H.xy));
            float t = atan(T.z / length(T.xy));
            nowAO = sin(h) - sin(t);
        #endif

        #ifdef HBAO_M2
            nowAO = clamp01(dot(H, normal));
        #endif

            float dis = -viewPos.z + nowViewPos.z;
            nowAO *= 1.0 - smoothstep(0.0, 5.0, dis);

            if(nowAO > maxAO){
                maxAO = nowAO;
            }
        }
        ao += maxAO;
    }
    ao /= dirCount;
    ao = pow(ao * 1, 1.0);

    return clamp01(ao);
}



// vec3 ssaoSamples[16] = vec3[](
//     vec3( 0.5381,  0.1856,  0.7812),
//     vec3( 0.1379,  0.2486,  0.9598),
//     vec3( 0.3371,  0.5679,  0.7506),
//     vec3( 0.6657, -0.0480,  0.7449),
//     vec3( 0.6715,  0.6624,  0.3343),
//     vec3( 0.6974, -0.3925,  0.6011),
//     vec3( 0.6898, -0.5983,  0.4084),
//     vec3(-0.6331, -0.0063,  0.7740),
//     vec3(-0.3951,  0.5719,  0.7176),
//     vec3(-0.8676,  0.3855,  0.3122),
//     vec3(-0.6776,  0.6408,  0.3597),
//     vec3(-0.9645,  0.1031,  0.2423),
//     vec3(-0.5401, -0.7666,  0.3490),
//     vec3( 0.1178, -0.9566,  0.2654),
//     vec3( 0.7310, -0.6761,  0.0948),
//     vec3(-0.2419, -0.9708,  0.0000)
// );

float SSAO(vec4 viewPos, vec3 normal){
    float noise = rand2_1(texcoord + sin(frameTimeCounter));
    vec3 randomVec = rand2_3(texcoord + sin(frameTimeCounter)) * 2.0 - 1.0;
    // randomVec = vec3(1.0, 0.0, 0.0);

    vec3 tangent = normalize(randomVec - normal * dot(randomVec, normal));
    vec3 bitangent = normalize(cross(normal, tangent));
    mat3 TBN = mat3(tangent, bitangent, normal);

    const float sampleCount = 10.0;
    const float radius = 1.5;
    float dis = clamp(length(viewPos.xyz) / 20.0, 1.0, 3.0);

    float ao = 0.0;
    float c = 0.0;
    for(int i = 0; i < sampleCount / dis; ++i){
        float scale = float(i) / sampleCount;
        scale = lerp(0.1, 1.0, scale * scale);
        // float scale = 1.0;
        // vec3 offset = ssaoSamples[i] * scale;
        vec3 offset = rand2_3(texcoord + sin(frameTimeCounter) + i);
        offset.xy = offset.xy * 2.0 - 1.0;
        offset.xy *= scale;
        // offset.xy = rotate2D(noise * _2PI) * offset.xy;
        offset = TBN * offset;

        vec3 samplePos = viewPos.xyz + offset * radius;
        samplePos = viewPosToScreenPos(vec4(samplePos, 1.0)).xyz;
        float nowDepth = texture(depthtex1, samplePos.xy).r;

        nowDepth = expDepthToLinearDepth(nowDepth);
        samplePos.z = expDepthToLinearDepth(samplePos.z);

        if(nowDepth < samplePos.z){
            float weight = 1.0;

            float rangeCheck = smoothstep(0.0, 1.0, 1.0 / (samplePos.z - nowDepth));
            weight *= rangeCheck;
            
            if(outScreen(samplePos.xy))
                weight = 0.0;

            float nowAO = 1.0;
            ao += nowAO * weight; 
            // c += weight;
        }
        c += 1.0;
    }
    ao /= c;
    ao = pow(ao * 1.0, 1.45);
    return clamp01(mix(1.0, 1.0 - ao, 1.0));
}