

float blockerSearch(sampler2D shadowMap, vec3 shadowPos, float radius, float quality, float dReceiver){
    float c = 0.0;
    float blocker = 0.0;

    #define SM_M2
#ifdef SM_M1
    // 方形采样
    float d = 2.0 * radius / quality;
    for(float i = -radius; i < radius + 0.1; i += d){
    for(float j = -radius; j < radius + 0.1; j += d){
        vec2 offset = vec2(i, j) / shadowMapResolution;
        float dBlocker = textureLod(shadowMap, shadowPos.xy + offset, log2(d)).r;
        if(dReceiver > dBlocker){
            blocker += (dReceiver - dBlocker) / dBlocker;
            c++;
        }
    }
    }
#endif

#ifdef SM_M2
    // 每次循环跨越的角度和距离
    float ANGLE_STEP = 3.883222077450933; // (sqrt(5)-1)/2 * 2PI
    float radiusStep = radius;

    // 设置起点（初始角度，距离），应用noise
    float noise = rand2_1(texcoord + 1.0 * sin(1.0 * frameTimeCounter));
    float angle = noise * _2PI;
    radius *= noise;

    for (int i = 0; i < quality; i++) {
        vec2 offset = vec2(cos(angle), sin(angle)) * pow(radius, 0.75);
        float dBlocker = textureLod(shadowMap, shadowPos.xy + offset / shadowMapResolution, log2(radiusStep)).r;
        if(dReceiver > dBlocker){
            blocker += (dReceiver - dBlocker) / dBlocker;
            c++;
        }
        radius += radiusStep;
        angle += ANGLE_STEP;
    }
#endif

    if(c <= 0.8) return -1.0;
    return blocker / c;
}

float PCF(sampler2DShadow shadowMap, vec3 shadowPos, float radius, float quality){
    float c = 0.0;
    float shade = 0.0;

#define SM_M2
#ifdef SM_M1
    // 方形采样
    float d = 2.0 * radius / quality;
    for(float i = -radius; i < radius + 0.1; i += d){
    for(float j = -radius; j < radius + 0.1; j += d){
        float noise = rand2_1(texcoord + 1.0 * sin(1.0 * frameTimeCounter));
        // vec3 noise = rand2_3(texcoord + vec2(i, j));
        vec2 offset = vec2(i, j);

        offset += vec2(noise.xx - 0.5) * d;
        offset = offset * rotate2D(noise * _2PI);

        offset /= shadowMapResolution;
        shade += texture(shadowMap, vec3(shadowPos.xy + offset, shadowPos.z), 0.0).r;
        c++;
    }
    }
#endif

#ifdef SM_M2
    // 每次循环跨越的角度和距离
    float ANGLE_STEP = 3.883222077450933; // (sqrt(5)-1)/2 * 2PI
    float radiusStep = radius;

    // 设置起点（初始角度，距离），应用noise
    float noise = rand2_1(texcoord + 1.0 * sin(1.0 * frameTimeCounter));
    float angle = noise * _2PI;
    radius *= noise;

    for (int i = 0; i < quality; i++) {
        vec2 offset = vec2(cos(angle), sin(angle)) * pow(radius, 0.75);
        shade += texture(shadowMap, vec3(shadowPos.xy + offset / shadowMapResolution, shadowPos.z)).r;
        c++;
        radius += radiusStep;
        angle += ANGLE_STEP;
    }
#endif

    return smoothstep(0.0, 1.0, pow(shade / c, 0.85));
}

float shadowMapping(vec4 worldPos, vec3 normal){
    if(skyB > 0.5) return 1.0;
    float worldDis = length(worldPos.xyz);
    float distFactor = clamp((worldDis / 20), 1.0, 3.0);
    // 阴影空间
    vec4 shadowPos = getShadowPos(worldPos);
    // 平均遮挡深度
    float dReceiver = shadowPos.z;
    float blocker = blockerSearch(shadowtex0, shadowPos.xyz, 1.0, 9.0 / distFactor, dReceiver);
    float penumbra = max(0.0, blocker);

    // 确定PCF半径
    float radius = pow(penumbra * 150, 1.0);    // 根据面光源大小
    // if(plants > 0.5) radius = (1.0 - clamp01(worldDis / 90));
    if(plants > 0.5) radius = 1.0;
    radius = clamp(radius, 0.15, 9.0);
    float quality = 9.0;   // 采样次数
    quality = quality / distFactor;

    // 采样坐标偏移
    worldPos.xyz += normal * 0.05;
    // worldPos.xyz += normal * 0.03 * (1.0 - 1.0 * clamp01(worldDis / 60));
    shadowPos = getShadowPos(worldPos);
    shadowPos.z -= 0.00006;

    // PCF
    float shade = PCF(shadowtex1, shadowPos.xyz, radius, quality);

    return clamp01(shade) * (1.0 - rainStrength * 0.9);
}