// 来自 SEUS
vec2 offsetCoord2(vec2 uv, vec2 offset){
    ivec2 texSize = textureSize(tex, 0);
    uv = uv * texSize;
    vec2 parallaxOffset = uv + mod(offset * texSize, vec2(TEXTURE_RESOLUTION));

	vec2 minCoord = vec2(uv.x - mod(uv.x, TEXTURE_RESOLUTION), uv.y - mod(uv.y, TEXTURE_RESOLUTION));
	vec2 maxCoord = minCoord + TEXTURE_RESOLUTION;

    if (parallaxOffset.x > maxCoord.x) {
		parallaxOffset.x -= TEXTURE_RESOLUTION;
	} else if (parallaxOffset.x < minCoord.x) {
		parallaxOffset.x += TEXTURE_RESOLUTION;
	}

	if (parallaxOffset.y > maxCoord.y) {
		parallaxOffset.y -= TEXTURE_RESOLUTION;
	} else if (parallaxOffset.y < minCoord.y) {
		parallaxOffset.y += TEXTURE_RESOLUTION;
	}
    
    return parallaxOffset / texSize;
}

// 来自 滑稽菌 的 Lunar Drips
vec2 offsetCoord1(vec2 coord) {
    return mod(coord - v_tcrange.xy, v_tcrange.zw) + v_tcrange.xy;
}

// 来自 BSL shader
vec2 offsetCoord(vec2 coord){
    return fract(coord) * vTexCoordAM.pq + vTexCoordAM.st;
}

float getParallaxHeight(vec2 uv, vec2 texGradX, vec2 texGradY){
    return mix(1.0, textureGrad(normals, uv, texGradX, texGradY).a, 1.0);
}

// 效果设计-Parallax Mapping视差映射
// https://miusjun13qu.feishu.cn/docx/G17IdiCyhoEd7XxBqJOcb3J1nie
vec2 parallaxMapping(vec2 coord, vec3 viewVector, vec3 lightDirTS, vec2 texGradX, vec2 texGradY, out vec3 parallaxOffset, out vec3 parallaxOffset1){
    float dist = length(vViewPos.xyz);
    if(dist > 20.0) return coord;
    
    const float slicesMin = 60.0;
    const float slicesMax = 120.0;
    const float height = PARALLAX_HEIGHT;

    float slicesNum = ceil(lerp(slicesMax, slicesMin, abs(dot(vec3(0, 0, 1), viewVector))));
#ifdef VOXEL_PARALLAX
    slicesNum = max(slicesNum / (dist * 0.75), 30.0);
#endif
    float dHeight = 1.0 / slicesNum;
    vec2 dUV = height * (viewVector.xy / viewVector.z) / slicesNum;
    
    vec2 currUVOffset = vec2(0.0);
    float weight = 0.0;

    float prevHeight = getParallaxHeight(offsetCoord(vTexCoord), texGradX, texGradY);
    float currHeight = prevHeight;
    float rayHeight = 1.0;
    if(prevHeight < 1.0){
        rayHeight = 1.0 - dHeight;
        currUVOffset -= dUV;
        currHeight = getParallaxHeight(offsetCoord(vTexCoord + currUVOffset), texGradX, texGradY);
        for(int i = 0; i < slicesNum; ++i){
            if(currHeight > rayHeight){
                break;
            }
            prevHeight = currHeight;
            currUVOffset -= dUV;
            rayHeight -= dHeight;
            currHeight = getParallaxHeight(offsetCoord(vTexCoord + currUVOffset), texGradX, texGradY);
        }

        float currDeltaHeight = currHeight - rayHeight;
        float prevDeltaHeight = rayHeight + dHeight - prevHeight;
        weight = currDeltaHeight / (currDeltaHeight + prevDeltaHeight);
    }

    vec2 lerpOffset = weight * dUV;
    parallaxOffset = vec3(currUVOffset + dUV/** sign(dot(lightDirTS, viewVector)+ 0.1)*/, rayHeight + dHeight);

#ifdef VOXEL_PARALLAX
    // lerpOffset = -weight * dUV * 1.0;
    lerpOffset = vec2(0.0);
#endif
    parallaxOffset1 = vec3(currUVOffset + lerpOffset, rayHeight);
    return offsetCoord(vTexCoord + parallaxOffset1.st);
}

float parallaxShadow(vec3 parallaxOffset, vec3 lightDirTS, vec2 texGradX, vec2 texGradY){
    float distance = length(vViewPos.xyz);
    if(distance > 20.0) return 1.0;

    const float slicesMin = 8.0;
    const float slicesMax = 16.0;
    const float height = PARALLAX_HEIGHT;
    const float shadowSoftening = 1.0;

    float slicesNum = ceil(lerp(slicesMax, slicesMin, abs(dot(vec3(0, 0, 1), lightDirTS))));
    // float parallaxHeight = getParallaxHeight(offsetCoord(vTexCoord + parallaxOffset.st), texGradX, texGradY);
    float parallaxHeight = parallaxOffset.z;
    float dDist = 1.0 / slicesNum;
    float dHeight = (1.0 - parallaxHeight) / slicesNum;
    vec2 dUV = height * dHeight * lightDirTS.xy / lightDirTS.z;

    vec2 currUVOffset = parallaxOffset.st + dUV;
    float rayHeight = parallaxHeight + dHeight;
    float dist = dDist;
    float currHeight = getParallaxHeight(offsetCoord(vTexCoord + currUVOffset), texGradX, texGradY);

    float shadow = 0.0;
    if(parallaxHeight < 1.0){
        for (int i = 1; i < slicesNum && rayHeight < 1.0; i++){
            if (currHeight > rayHeight){
                shadow = max(shadow, (currHeight - rayHeight) / dist * shadowSoftening);
                if(1.0 == shadow) break;
                // shadow = 1.0;
                // break;
            }else{
                // shadow = max(shadow, (currHeight - rayHeight) / dist * shadowSoftening);
            }
            rayHeight += dHeight;
            currUVOffset += dUV;
            dist += dDist;
            currHeight = getParallaxHeight(offsetCoord(vTexCoord + currUVOffset), texGradX, texGradY);
        }
    }

    return clamp01(1.0 - shadow);
}