float fakeCaustics(vec3 pos){
    float height = 64.0;

    // 点积向上向量和光照方向向量得到cos值   预定高度和实际高度的高度差
    float cosUpSunpos = abs(dot(vec3(0.0,1.0,0.0), lightWorldDir));
    float hDiff = abs(height - pos.y);

    // 高度差 * （1除以cos）得到斜边长度   sqrt（斜边的平方-邻边的平方）得到对边长度
    float hyp = hDiff * (1 / cosUpSunpos + 0.01);
    float dist = sqrt(hyp * hyp - hDiff * hDiff);

    // 单位化光照向量，乘上对边长度得到偏移向量
    vec3 unit = normalize(vec3(lightWorldDir.x, 0.0, lightWorldDir.z));
    vec3 offset = dist * unit;

    vec2 waveUV = vec2(0.0);
    if(pos.y < 64){
        waveUV = pos.xz + offset.xz;
    }else{
        waveUV = pos.xz - offset.xz;
    }

    // worley 伪造焦散，最后用pow值调整曲线
    float caustics  = worley(waveUV * 0.75 + vec2(0.5 * frameTimeCounter), 1.0) * 0.6;
          caustics += worley(waveUV * 1.5 - vec2(0.5 * frameTimeCounter), 1.0) * 0.4;
        //   caustics += worley(waveUV * 4.0 + vec2(0.5 * frameTimeCounter), 1.0) * 0.2;
        //   caustics += worley(waveUV * 8.0 - vec2(0.5 * frameTimeCounter), 1.0) * 0.1;
          caustics = pow(1.0 - caustics, 2.0);

    // 波浪高度伪造焦散，mix调整权重
    caustics = mix(caustics, 1.0, 0.2);

    // 控制焦散强度曲线
    caustics = pow(caustics, 1.0);
    caustics = mix(1.0, caustics * 1.0, 1.00);

    return caustics;
}

vec3 waterFog(vec3 color, float waterDis, vec3 ambiColor, float waterLightmap){
    
    vec3 fogColor = waterFogColor * 1.00 * getLuminance(ambiColor) * waterLightmap;
    if(waterA > 0.5 && isEyeInWater < 0.5){
        float mixFactor = smoothstep(0.0, 15.0, waterDis);
        mixFactor = mix(1.0, pow(mixFactor, 0.30), 0.5);
        color = mix(color, fogColor, mixFactor);
    }

    return color;
}

vec3 underWaterFog(vec3 color, vec3 worldDir, float worldDis, vec3 waterFogColorUsed){
    const float numCount = 5;
    float stepSize = 3.0;
    vec3 stepVec = stepSize * worldDir;

    const float dDensity = 60.0;
    float density = 0.0;
    int c = 0;
    for(int i = 0; i < numCount; ++i){
        vec3 nowPos = vec3(0.0) + i * stepVec;
        nowPos += BlueNoise(texcoord, 64.0) * stepVec * 1.0;
        if(length(nowPos) > worldDis) break;

        vec3 shadowPos = getShadowPos(vec4(nowPos, 1.0)).xyz;
        float shade = texture(shadowtex1, shadowPos);

        if(shade > 0.5){
            density += pow(fakeCaustics(nowPos + cameraPosition), 2.2); 
        }
        ++c;
    }
    density *= dDensity;
    density /= c + 0.01;
    const float g = 0.45;
    float g2 = g * g;
    density *= (1 - g2) / (4.0 * PI * pow((1 + g2 - 2 * g * dot(sunWorldDir, worldDir)), 3.0 / 2.0));

    waterFogColorUsed *= (0.5 + density) * (eyeBrightnessSmooth.y / 240.0 + 0.1);
    color.rgb = mix(color, waterFogColorUsed, pow(clamp01(worldDis / 40.0), 0.25));

    color += rand2_1(texcoord + sin(frameTimeCounter)) / 512.0;
    return color;
}