// 折射参考了 SEUS
vec3 waterRefraction(vec3 color, vec3 waterViewNormal, float worldDis0, float worldDis1){
    float waterDepth = clamp01((worldDis1 - worldDis0) / far);
    // float refractAmount =  clamp(0.2 * waterDepth, 0.0, 0.025);
    float refractAmount =  1.5;
    // vec3 waterWorldNormal = getWaterNormal(mcPos, 0.0).xzy;
    // vec3 waterViewNormal = normalize((gbufferModelView * (vec4(waterWorldNormal, 0.0))).xyz);

    vec3 waterOriNormal = normalize(normalDecode(texture(colortex3, texcoord).rg));

    // vec3 upViewDir = normalize((gbufferModelView * vec4(upWorldDir, 0.0)).xyz);
    waterViewNormal.xy -= waterOriNormal.xy;


    // vec2 refractCoord0 = texcoord - waterNormal.xy * refractAmount / (worldDis0 / far + 0.0001);
    const float refractAmountMax = 1.0;
    vec2 refractCoord0 = texcoord - clamp(waterViewNormal.xy * refractAmount / (worldDis0 + 0.0001), vec2(-refractAmountMax), vec2(refractAmountMax));
    if(outScreen(refractCoord0)) 
        refractCoord0 = texcoord;
    // if(abs(waterWorldNormal.y) < 0.9)
    //     return color;
    
    color = textureLod(colortex0, refractCoord0.xy, 0.0).rgb * 1.0;
    color += textureLod(colortex0, refractCoord0.xy, 1.0).rgb * 0.5;
    color += textureLod(colortex0, refractCoord0.xy, 2.0).rgb * 0.25;

    return color;
}

vec3 waterSkyReflection(vec3 worldDir, vec3 reflectDir, vec3 waterNormal, float waterLightmap){
    vec3 reflectColor = AtmosphericScattering(reflectDir, sunWorldDir, 5.0, 5.0);

    vec3 halfVecWS = normalize(sunWorldDir - worldDir);
    float NdotHS = max(dot(waterNormal, halfVecWS), 0.0);

    vec3 specular = pow(NdotHS, 5.0) * sunColor * 1.00;
    // specular += pow(NdotHS, 4000.0) * sunColor * 100.0;
    reflectColor *= 1.0 + specular;
    reflectColor += drawCelestial(reflectDir, 1.0) * smoothstep(0.8, 1.0, waterLightmap);

    // float cosTheta = dot(-worldDir, upWorldDir);
    // float F0 = 0.02;
    // float fresnel = mix(pow(1.0 - clamp01(cosTheta), FRESNEL_POWER), 1.0, F0);

    return reflectColor;
}

vec3 waterReflection(vec3 color, vec3 startPos, vec3 worldDir, vec3 waterViewNormal, vec3 waterWorldNormal, float waterLightmap){
    // 初始化反射的颜色为天空色
    vec3 reflectColor = color;
    if(isEyeInWater == 0){
        vec3 reflectWorldDir = normalize(reflect(worldDir, waterWorldNormal));
        if(reflectWorldDir.y < 0.0)	reflectWorldDir = normalize(vec3(reflectWorldDir.x, 0.0, reflectWorldDir.z));
        reflectColor = waterLightmap * waterSkyReflection(worldDir, reflectWorldDir, waterWorldNormal, waterLightmap);
    }

    vec3 viewDir = normalize(startPos);
    vec3 reflectViewDir = normalize(reflect(viewDir, waterViewNormal));
    float stepSize = 0.25;
    vec3 stepVec = reflectViewDir * stepSize;

    vec3 waterOriNormal = normalize(normalDecode(texture(colortex3, texcoord).rg));
    vec3 testPos = startPos;    
    testPos += waterOriNormal * 0.1; 

    bool hit = false;
    // 反射方向朝上，且远离视线方向时 进行碰撞检测
    if(/*(dot(reflectViewDir, waterOriNormal) > 0.0 && dot(reflectViewDir, normalize(startPos)) > 0.0) 
        || (isEyeInWater == 1)*/ true){
        vec3 testScreenPos;
        for(float i = 1.0; i < 20; ++i){
            float powV = 1.6;
            i = i + BlueNoise(texcoord, 64.0);
            vec3 ds = stepVec * pow(i, powV);
            testPos += ds; 
            testScreenPos = viewPosToScreenPos(vec4(testPos, 1.0)).xyz;
            if(outScreen(testScreenPos.xy) || testScreenPos.z < 0.0){
                hit = true;
                break;  
            }

            // 初次碰撞
            float closest = texture(depthtex0, testScreenPos.xy).r;
            if(testScreenPos.z > closest){
                hit = true;
                // 二分法精确碰撞位置
                float sig = -1.0;
                for(int j = 1; j < 5; ++j){
                    float n = pow(0.5, float(j));
                    testPos = testPos + sig * n * ds;
                    testScreenPos = viewPosToScreenPos(vec4(testPos, 1.0)).xyz;
                    closest = texture(depthtex0, testScreenPos.xy).r;
                    sig = sign(closest - testScreenPos.z);
                }
                // 根据阙值（碰撞点和测试点位置的差距）确定是否更新反射颜色
                vec3 newTestPos = screenPosToViewPos(vec4(vec3(testScreenPos.xy, closest), 1.0)).xyz;
                float dist = clamp(pow(i, powV) * stepSize * length(startPos) / 5.0, 0.0, 10.0);
                float distTest = length(testPos - newTestPos);
                if(distTest < 0.1 + dist){
                    reflectColor = texture(colortex0, testScreenPos.xy).rgb;
                }
                break;
            }
        }
        // 若没有碰撞，则直接使用当前位置作为反射颜色
        if(!hit){
            reflectColor = texture(colortex0, testScreenPos.xy).rgb;
        }
    }
    
    // 使用菲涅尔确定反射强度
    float F0 = 0.02;
    float cosTheta = dot(-worldDir, waterWorldNormal);
    float fresnel = mix(pow(1.0 - clamp01(cosTheta), FRESNEL_POWER - 0.0 * isEyeInWater), 1.0, F0);

    color = mix(color, reflectColor, fresnel);

    return color;
}