varying vec2 texcoord;
varying vec3 sunWorldDir, moonWorldDir, lightWorldDir;

varying vec3 sunColor, nightColor, lightColor, ambiColor;

varying float sunRiseSet, isNoon, isNight;

#include "/lib/uniform.glsl"
#include "/lib/setting.glsl"
#include "/lib/common/common.glsl"
#include "/lib/camera/colorToolkit.glsl"
#include "/lib/common/position.glsl"
#include "/lib/common/noise.glsl"
#include "/lib/common/normal.glsl"

#ifdef FSH
const bool shadowtex0Mipmap = true;
const bool shadowtex0Nearest = false;

const bool shadowcolor0Mipmap = true;
const bool shadowcolor0Nearest = false;
const bool shadowcolor1Mipmap = true;
const bool shadowcolor1Nearest = false;

#define CPS_FSH
#include "/lib/camera/filter.glsl"
#include "/lib/TAA.glsl"
// #include "/lib/common/materialIdMapper.glsl"
#include "/lib/lighting/mcLightmap.glsl"
#include "/lib/lighting/RSM.glsl"
#include "/lib/lighting/SSAO.glsl"


void main() {
	vec3 color = texture(colortex0, texcoord).rgb;
    gammaToLinear(color.rgb);

	vec2 ambientOffset = vec2(0.0);
	vec2 ambientUV = texcoord * exp2(1.0);
	float ambientDepth = texture(depthtex1, ambientUV).r;
	// ambientUV = unTAAJitterTexcoord(ambientUV);

	vec4 ambientScreenPos = vec4(unTAAJitterTexcoord(ambientUV), ambientDepth, 1.0);
	vec4 ambientViewPos = screenPosToViewPos(ambientScreenPos);
	vec4 ambientWorldPos = viewPosToWorldPos(ambientViewPos);

	vec3 ambient = BLACK;
	// vec4 CT5 = texture(colortex5, texcoord);
	if(!outScreen(ambientUV) && ambientDepth < 1.0){
		// lightmap
		float mcLightmap = texture(colortex4, ambientUV).r;

		// mc lightmap color
		vec3 ambiColor = mix(vec3(getLuminance(ambiColor)), ambiColor, 0.5);
		ambient = max(mcLightmap * ambiColor, 0.1 * nightVision) * 4.0;

		float CT7 = unpack16Bit(texture(colortex7, ambientUV).b).y;
    	float isWater = (CT7 > 0.2/*||CT6_A > 0.90*/) ? 1.0 : 0.0;
		// rsm color
		float worldDis1 = length(ambientWorldPos);
		vec3 hrrNormal = getNormal(ambientUV);
		vec3 hrrNormalW = normalize((gbufferModelViewInverse * vec4(hrrNormal, 0.0)).xyz);
		if(isEyeInWater == 0  && worldDis1 < 30 && getLuminance(sunColor) > 0.0 && rainStrength < 0.99 && isWater < 0.5){
			vec3 rsmColor = RSM(hrrNormalW, ambientWorldPos);
			ambient += (1.0 - rainStrength) * rsmColor * sunColor * 15.0 * smoothstep(0.0, 10.0, 30 - worldDis1);
		}
		// SSAO
		float ao = SSAO(ambientViewPos, hrrNormal);
		ambient *= ao;
		
		// CT5.r += ambientDepth;
	}
	float parallaxShadow = texture(colortex4, texcoord).a;

	ambient = max(ambient, BLACK);
	color.rgb = max(BLACK, color.rgb);

/* DRAWBUFFERS:04 */
	gl_FragData[0] = vec4(color, 1.0);
	gl_FragData[1] = vec4(ambient, parallaxShadow);
	// gl_FragData[2].rg = CT5.rg;
}


#endif
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef VSH
#include "/lib/atmosphere/atmosphericScattering.glsl"

void main() {
	sunWorldDir = normalize(viewPosToWorldPos(vec4(sunPosition, 0.0)).xyz);
    moonWorldDir = normalize(viewPosToWorldPos(vec4(moonPosition, 0.0)).xyz);
    lightWorldDir = normalize(viewPosToWorldPos(vec4(shadowLightPosition, 0.0)).xyz);

	isNoon = clamp01(dot(sunWorldDir, upWorldDir) * NOON_DURATION);
	isNight = clamp01(dot(moonWorldDir, upWorldDir) * NIGHT_DURATION);
	sunRiseSet = clamp01(1 - isNoon - isNight);

	sunColor = Transmittance1(vec3(0.0, camera_h, 0.0), sunWorldDir, 5.0);
	sunColor = mix(sunColor, vec3(getLuminance(sunColor)), rainStrength * 0.8);
	nightColor = isNight * Transmittance1(vec3(0.0, camera_h, 0.0), moonWorldDir, 5.0) * NIGHT_LIGHT_BRI;
	lightColor = sunColor + nightColor;
	ambiColor = AtmosphericScattering(upWorldDir, sunWorldDir, 5.0, 3.0)
			+ isNight * nightColor * NIGHT_AMBIENT_BRI;


	gl_Position = ftransform();
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
}
#endif
