varying vec2 texcoord;
varying vec3 sunWorldDir, moonWorldDir, lightWorldDir;
varying vec3 sunViewDir, moonViewDir, lightViewDir;

varying vec3 sunColor, nightColor, lightColor, ambiColor;

varying float sunRiseSet, isNoon, isNight;


#include "/lib/uniform.glsl"
#include "/lib/setting.glsl"
#include "/lib/common/common.glsl"
#include "/lib/camera/colorToolkit.glsl"
#include "/lib/common/position.glsl"
#include "/lib/camera/filter.glsl"
#include "/lib/common/noise.glsl"
#include "/lib/atmosphere/atmosphericScattering.glsl"
#include "/lib/common/normal.glsl"

#ifdef FSH
const int RGBA8 = 0;
const int R16 = 0;
const int R16F = 0;
const int RG16 = 0;
const int RG16F = 0;
const int RGB16F = 0;
const int RGBA16 = 0;
const int RGBA16F = 0;
const int R32 = 0;
const int R32F = 0;
const int RG32F = 0;
const int RGB32F = 0;
const int RGBA32F = 0;

/*  colortex
*   0   rgba = main color
*   1   r = block ID, gbufferID, 		g = lightmap(gbf - cp1), 		ba = normal
*   2   rgb = preColor 			a = other temporal data
*   3   rg = water normal(gbf-cp4), velocity(cp7-cp10)
*   4   a = parallaxShadow(gbf-cp2), rgb = hrr RSM SSAO(CP1-2), hrr AS(CP2-3), hrr VL(CP2-3), hrr cloud(CP3-4), bloom sample(CP5-8)
*   5	r = hrr depth(CP1-4)	g = specular(gbf-cp2)
*   6   rgba = translucency color
*   7   rg = wave normal, 	b = translucency lightmap, water mark
*/

const int colortex0Format = RGBA16F;
const int colortex1Format = RGBA16;
const int colortex2Format = RGBA16F;
const int colortex3Format = RGBA16F;
const int colortex4Format = RGBA16F;
const int colortex5Format = RGBA16F;
const int colortex6Format = RGBA8;
const int colortex7Format = RGBA16;

const int shadowcolor0Format = RGBA8;
const int shadowcolor1Format = RGBA8;

const bool colortex4MipmapEnabled = true;

const bool shadowtex0Mipmap = true;
const bool shadowtex0Nearest = false;

const bool colortex2Clear = false;
const bool colortex5Clear = false;

#define CPS_FSH
#include "/lib/TAA.glsl"
#include "/lib/common/materialIdMapper.glsl"
#include "/lib/lighting/mcLightmap.glsl"
#include "/lib/lighting/shadowMapping.glsl"
#include "/lib/atmosphere/volumetricLighting.glsl"
#include "/lib/water/waterEffect.glsl"
// #include "/lib/lighting/SSAO.glsl"

vec3 getAmbiColor(vec3 normal, float linearDepth){
	vec2 uv = texcoord * pow(0.5, 1.0) + vec2(0.0, 0.0);
	vec3 oriColor = texture(colortex4, uv).rgb;

	float c = 0.0;
	vec3 ambient = vec3(0.0);
#ifdef RSM_M1
	const float radius = 0.5;
	const float quality = 1.0;
#endif

#ifdef RSM_M2
	const float radius = 0.5;
	const float quality = 1.0;
#endif

	float d = 2.0 * radius / quality;
	for(float i = -radius; i <= radius + 0.1; i += d){
	for(float j = -radius; j <= radius + 0.1; j += d){
		vec2 offset = vec2(i, j) / texResolution;
		vec2 newUV = uv + offset;

		// float nowDepth = texture(colortex5, newUV).r;
		float nowDepth = texture(depthtex1, newUV * 2.0).r;
		nowDepth = expDepthToLinearDepth(nowDepth);
		float weight = clamp(1.0f - abs(linearDepth - nowDepth) / 2.0f, 0.0, 1.0);
	
	#ifdef RSM_M2
		vec3 nowNormal = getNormal(newUV * 2.0);
		weight *= max(0.0f, dot(nowNormal, normal) * 2.0f - 1.0f);
	#endif

		vec3 nowColor = textureLod(colortex4, newUV, 0.0).rgb;
		// weight *= clamp(1.0 - distance(oriColor, nowColor), 0.0, 1.0);

		ambient += nowColor * weight;
		c += weight;
	}
	}
	ambient /= max(c, 0.01);
	return ambient;
}

vec3 clipAABB1(vec3 nowColor, vec3 preColor){
    //累计周围颜色值及平方值
    vec2 deltaRes = vec2(1.0 / viewWidth, 1.0 / viewHeight);
    vec3 m1 = vec3(0), m2 = vec3(0);

    for(int i = -1; i <= 1; i++){
    for(int j = -1; j <= 1; j++){
        vec2 newUV = texcoord.st + deltaRes * vec2(i, j);
        vec3 C = RGB2YCoCgR(ToneMap(textureLod(colortex4, newUV * 0.5, 0.0).rgb));
        m1 += C;
        m2 += C * C;
    }
    }

    //计算均值和标准差，以确定包围盒大小
    const float TAA_variance_clip_gamma = 10.0;
    vec3 aabbMin = nowColor, aabbMax = nowColor;
    const int N = 9;
    vec3 mu = m1 / N;
    vec3 sigma = sqrt(abs(m2 / N - mu * mu));
    aabbMin = mu - TAA_variance_clip_gamma * sigma;
    aabbMax = mu + TAA_variance_clip_gamma * sigma;

    //根据包围盒大小进行约束
    vec3 p_clip = 0.5 * (aabbMax + aabbMin);
    vec3 e_clip = 0.5 * (aabbMax - aabbMin);

    vec3 v_clip = preColor - p_clip;
    vec3 v_unit = v_clip.xyz / e_clip;
    vec3 a_unit = abs(v_unit);
    float ma_unit = max(a_unit.x, max(a_unit.y, a_unit.z));

    if (ma_unit > 1.0)
        return p_clip + v_clip / ma_unit;
    else
        return preColor;
}

void main() {
	vec4 color = texture(colortex0, texcoord);
	// depth
	float depth = texture(depthtex0, texcoord).r;
	float depth1 = texture(depthtex1, texcoord).r;
	float linearDepth = expDepthToLinearDepth(depth);
	float linearDepth1 = expDepthToLinearDepth(depth1);

	// coords
    vec4 viewPos = screenPosToViewPos(vec4(texcoord, depth1, 1.0));
    vec4 worldPos = viewPosToWorldPos(viewPos);
    vec3 mcPos = worldPos.xyz + cameraPosition;
    float worldDis = clamp(length(worldPos.xyz) / far, 0.0, 1.0);
    vec3 worldDir = normalize(worldPos.xyz);

	// TAA jitter coords
	vec2 tTexcoord = unTAAJitterTexcoord(texcoord);
	vec4 tViewPos = screenPosToViewPos(vec4(tTexcoord, depth1, 1.0));
	vec4 tWorldPos = viewPosToWorldPos(tViewPos);
	vec3 tWorldDir = normalize(tWorldPos.xyz);

	// block normal
	vec3 normal = normalize(normalDecode(texture(colortex1, texcoord).ba));
	vec3 normalW = normalize((gbufferModelViewInverse * vec4(normal, 0.0)).xyz);
	

	// vec3 ambient = vec3(0.0);
	vec3 RSM = vec3(0.0);
	if(skyB < 0.5){
		// lightmap
		vec2 mcLightmap = unpack16Bit(texture(colortex1, texcoord).g);
		mcLightmap = AdjustLightmap(mcLightmap);
	




		// plants normal
		vec3 N1 = normalW;
		if(plants > 0.5) N1 = vec3(0.0, 1.0, 0.0);

		// light dot normal
		float NdotL = max(dot(N1, lightWorldDir), 0.0);
		float NdotS = max(dot(N1, sunWorldDir), 0.0);
		float NdotM = max(dot(N1, moonWorldDir), 0.0);

		if(plants < 0.5){
			NdotL = pow(NdotL, 0.55);
		}else{
			NdotL = pow(NdotL, 0.20);
		}

		// normal dot half vector
		vec3 N2 = normalW;

		vec3 halfVecWL = normalize(lightWorldDir - worldDir);
		float NdotHL = max(dot(N2, halfVecWL), 0.0);

		vec3 halfVecWS = normalize(sunWorldDir - worldDir);
		float NdotHS = max(dot(N2, halfVecWS), 0.0);

		vec3 halfVecWM = normalize(moonWorldDir - worldDir);
		float NdotHM = max(dot(N2, halfVecWM), 0.0);
		




		// shadow and caustics
		float shade = shadowMapping(tWorldPos, normalW);
		// if(GBF_ENTITIES < 0.5){
			float parallaxShadow = texture(colortex4, texcoord).a;
			shade = min(shade, parallaxShadow);
		// }

		// water depth
		float dot_worldDir_upVec = abs(dot(worldDir, upWorldDir));
		float worldDis0 = length(viewPosToWorldPos(screenPosToViewPos(vec4(texcoord, depth, 1.0))));
		float worldDis1 = length(viewPosToWorldPos(screenPosToViewPos(vec4(texcoord, depth1, 1.0))));
    	float waterDepth = 0.0;
		if(waterA > 0.5 && isEyeInWater < 0.5){
			waterDepth = abs(worldDis1 - worldDis0) * dot_worldDir_upVec;
		}

		// caustics
		float caustics = 1.0;
		if(isEyeInWater < 0.5 && waterA > 0.5){
			caustics = mix(1.0,  fakeCaustics(mcPos),  mix(1.0, clamp01(waterDepth / 2.0), 0.2));
		}
		if(isEyeInWater > 0.5 && waterA < 0.5){
			caustics = fakeCaustics(mcPos);
		}
		shade = min(shade, caustics);
		if(mcLightmap.y == 0.0) shade = 0.0;
			




		// diffuse & shade
		float diffuse = NdotL;
		vec3 direct = min(diffuse, shade) * lightColor * 0.3;
		// water light
		if(isEyeInWater == 1){
			direct *= pow(mcLightmap.y, 0.45) * 1.25;
		}
		// artificial light
		if(glowingB > 0.5) mcLightmap.x *= 2.0;
		direct += mcLightmap.x * vec3(1.5, 0.58, 0.10) * 0.20;






		// // mc lightmap color
		// vec3 ambiColor = mix(vec3(getLuminance(ambiColor)), ambiColor, 0.5);
		// vec3 ambient = max(mcLightmap.y * ambiColor, 0.1 * nightVision) * 4.0;
		// // rsm color
		// if(isEyeInWater == 0  && worldDis1 < 60 && getLuminance(sunColor) > 0.0 && waterA < 0.5 && rainStrength < 0.99){
		// 	vec3 rsmColor = RSM(normalW, tWorldPos);
		// 	ambient += (1.0 - rainStrength) * rsmColor * sunColor * 5.0 /** (1.0 - ao)*/;
		// }
		RSM = getAmbiColor(normal, linearDepth1);
		vec2 offsetUV = clamp01(texcoord - (texcoord.st - getPrePos(worldPos)));
		vec3 preColor = texture(colortex5, offsetUV).rgb;
		RSM = RGB2YCoCgR(ToneMap(RSM));
		preColor = RGB2YCoCgR(ToneMap(preColor));
    	preColor = clipAABB1(RSM, preColor);
    	preColor = UnToneMap(YCoCgR2RGB(preColor));
    	RSM = UnToneMap(YCoCgR2RGB(RSM));
		RSM = mix(preColor, RSM, 0.05);
		vec3 ambient = RSM;
		// SSAO
		// float ao = HBAO(tTexcoord, depth, normal);
		// float ao = SSAO(viewPos, normal);
		// ambient *= ao;





		// specular
		vec3 specular = pow(NdotHS, 25.0) * sunColor * 0.15;

		color.rgb *= direct * (1.0 + specular) + ambient;





		// water fog
		float waterLightmap = unpack16Bit(texture(colortex7, texcoord).b).x;
		waterLightmap = AdjustLightmap(vec2(0.0, waterLightmap)).y;
		color.rgb = waterFog(color.rgb, worldDis1 - worldDis0, ambiColor, waterLightmap);
		if(waterA > 0.5) color *= 1.25;
	}

	



	// sky color
	vec2 skyUV = texcoord * exp2(2.0);
	float skyDepth = texture(depthtex1, skyUV).r;
	skyUV = unTAAJitterTexcoord(skyUV);
	vec3 skyScreenPos = vec3(skyUV, skyDepth);
	vec4 skyViewPos = screenPosToViewPos(vec4(skyScreenPos, 1.0));
	vec4 skyWorldPos = viewPosToWorldPos(skyViewPos);
	vec3 skyWorldDir = normalize(skyWorldPos.xyz);

	vec4 CT4 = vec4(BLACK, 1.0);
	if(!outScreen(skyUV)){
		CT4.rgb += AtmosphericScattering(skyWorldDir, sunWorldDir, 15.0, 10.0);
		CT4.rgb += max(BLACK, AtmosphericScattering(skyWorldDir, moonWorldDir, 10.0, 3.0) * NIGHT_LIGHT_BRI * 0.5);
	}



	// volumetric light
	vec2 vlUV = (texcoord - vec2(0.5, 0.5)) * exp2(1.0);
	float vlDepth = texture(depthtex1, vlUV).r;
	// vlUV = (tTexcoord - vec2(0.5, 0.5)) * exp2(1.0);
	vlUV = unTAAJitterTexcoord(vlUV);
	vec4 vlViewPos = screenPosToViewPos(vec4(vlUV, vlDepth, 1.0));
	vec4 vlWorldPos = viewPosToWorldPos(vlViewPos);

	if((!outScreen(vlUV) && vlDepth < 1.0)) {
		CT4.rgb += volumetricLighting(vlUV, vlWorldPos.xyz, sunWorldDir);
	}

	// vec4 CT5 = vec4(0.0);
	// vec2 vlDepthUV = (texcoord - vec2(0.5, 0.5)) * exp2(1.0);
	// if(!outScreen(vlDepthUV)){
	// 	vlDepth = texture(depthtex1, vlDepthUV).r;
	// 	CT5.r += vlDepth;
	// }



	// color.rgb = texture(colortex4, texcoord).rgb;
	// color.rgb = unpackFloatToRGB(texture(colortex5, texcoord).g);
	// color.rgb = normalW;
	color.rgb = max(BLACK, color.rgb);

/* DRAWBUFFERS:045 */
	gl_FragData[0] = color;
	gl_FragData[1] = CT4;
	gl_FragData[2].rgb = RSM;
}


#endif
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef VSH

void main() {
	sunViewDir = normalize(sunPosition);
	moonViewDir = normalize(moonPosition);
	lightViewDir = normalize(sunPosition);

	sunWorldDir = normalize(viewPosToWorldPos(vec4(sunPosition, 0.0)).xyz);
    moonWorldDir = normalize(viewPosToWorldPos(vec4(moonPosition, 0.0)).xyz);
    lightWorldDir = normalize(viewPosToWorldPos(vec4(shadowLightPosition, 0.0)).xyz);

	isNoon = clamp01(dot(sunWorldDir, upWorldDir) * NOON_DURATION);
	isNight = clamp01(dot(moonWorldDir, upWorldDir) * NIGHT_DURATION);
	sunRiseSet = clamp01(1 - isNoon - isNight);

	sunColor = isNoon * Transmittance1(vec3(0.0, camera_h, 0.0), sunWorldDir, 5.0);
	sunColor = mix(sunColor, vec3(getLuminance(sunColor)), rainStrength * 0.8);
	nightColor = isNight * Transmittance1(vec3(0.0, camera_h, 0.0), moonWorldDir, 5.0) * NIGHT_LIGHT_BRI;
	lightColor = sunColor + nightColor;
	ambiColor = isNoon * AtmosphericScattering(upWorldDir, sunWorldDir, 5.0, 3.0)
			+ isNight * nightColor * NIGHT_AMBIENT_BRI;
	// ambiColor = Transmittance1(vec3(0.0, camera_h, 0.0), upWorldDir, 5.0);

	gl_Position = ftransform();
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
}
#endif
