varying vec2 texcoord;

varying vec3 sunWorldDir, moonWorldDir, lightWorldDir;

varying vec3 sunColor, nightColor, lightColor, ambiColor;

varying float sunRiseSet, isNoon, isNight;

#include "/lib/uniform.glsl"
#include "/lib/setting.glsl"
#include "/lib/common/common.glsl"
#include "/lib/common/position.glsl"
#include "/lib/camera/colorToolkit.glsl"
#include "/lib/camera/filter.glsl"
#include "/lib/common/noise.glsl"
#include "/lib/atmosphere/atmosphericScattering.glsl"


#ifdef FSH
// const bool colortex0MipmapEnabled = true;
const bool colortex4MipmapEnabled = true;

#define CPS_FSH
#include "/lib/TAA.glsl"
#include "/lib/common/materialIdMapper.glsl"
#include "/lib/atmosphere/volumetricLighting.glsl"
#include "/lib/atmosphere/volumetricClouds.glsl"

void main() {
	vec3 color = texture(colortex0, texcoord).rgb;

	// TAA Jitter coords
	float depth = texture(depthtex1, texcoord).r;

	vec4 viewPos = screenPosToViewPos(vec4(texcoord, depth, 1.0));
    vec4 worldPos = viewPosToWorldPos(viewPos);
    vec3 mcPos = worldPos.xyz + cameraPosition;
    float worldDis = clamp(length(worldPos.xyz) / far, 0.0, 1.0);
    vec3 worldDir = normalize(worldPos.xyz);

	vec2 tTexcoord = unTAAJitterTexcoord(texcoord);
	vec4 tViewPos = screenPosToViewPos(vec4(tTexcoord, depth, 1.0));
	vec4 tWorldPos = viewPosToWorldPos(tViewPos);
	vec3 tWorldDir = normalize(tWorldPos.xyz);



	// sky color & fog color
	if(skyB > 0.5){
		// AtmosphericScattering
		// GaussianBlurSimple Settings: text, uv, direction, radius, blurSize
		// vec3 AS = GaussianBlurSimple(colortex4, texcoord * pow(0.5, 2.0), 3.0, 2.0, 3.0).rgb;
		vec3 AS = textureNice(colortex4, texcoord * pow(0.5, 2.0), texResolution, 0.0);
		AS += vec3(mod(rand2_1(texcoord + sin(frameTimeCounter)) * 2.0f, 2.0f) - 1.0) / (1024.0 * (1.0 + 2.0 * isNight));
		color.rgb = AS;
	}else{
		// basic fog
		color.rgb = mix(color, ambiColor, pow(smoothstep(0.45, 1.0, worldDis), 0.75) * 0.15);
		// volumetric light
		color.rgb = getVolumetricLighting(color.rgb, depth, tWorldPos.rgb);
	}





	vec4 CT4 = vec4(0.0);
	vec4 CT5 = vec4(0.0);
	// CT5 = vec4(0.0, CT5.gba);

	// cloud color
	vec2 cloudOffset = vec2(0.0, 0.0);
	vec2 cloudUV = (texcoord - cloudOffset) * exp2(1.0);
	float cloudDepth = texture(depthtex1, cloudUV).r;
	// cloudUV = (tTexcoord - cloudOffset) * exp2(1.0);
	cloudUV = unTAAJitterTexcoord(cloudUV);
	vec4 cloudViewPos = screenPosToViewPos(vec4(cloudUV, cloudDepth, 1.0));
	vec4 cloudWorldPos = viewPosToWorldPos(cloudViewPos);
	vec3 cloudWorldDir = normalize(cloudWorldPos.xyz);

	if(!outScreen(cloudUV) && cloudDepth == 1.0) {
		vec4 cloudColor = vec4(BLACK, 1.0);
		vec4 cloudColorNight = vec4(BLACK, 1.0);
		if(getLuminance(sunColor) > 0.00)
			cloudRayMarching(vec3(0.0), cloudWorldDir, sunWorldDir, sunColor, 12.0, cloudColor);
		if(getLuminance(nightColor) > 0.00)
			cloudRayMarching(vec3(0.0), cloudWorldDir, moonWorldDir, nightColor * 0.25, 6.0, cloudColorNight);
		CT4 += vec4(cloudColor.rgb, 0.0) + vec4(cloudColorNight.rgb, 0.0) + vec4(BLACK, min(cloudColor.a, cloudColorNight.a));
	}

	// vec2 cloudDepthUV = (texcoord - cloudOffset) * exp2(1.0);
	// if(!outScreen(cloudDepthUV)){
	// 	cloudDepth = texture(depthtex1, cloudDepthUV).r;
	// 	CT5.r += cloudDepth;
	// }

	// color.rgb = texture(colortex4, texcoord).rgb;
	

	color.rgb = max(BLACK, color.rgb);

/* DRAWBUFFERS:04 */
	gl_FragData[0] = vec4(color, 1.0);
	gl_FragData[1] = CT4;
	// gl_FragData[2] = CT5;
}


#endif
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef VSH

void main() {
	sunWorldDir = normalize(viewPosToWorldPos(vec4(sunPosition, 0.0)).xyz);
    moonWorldDir = normalize(viewPosToWorldPos(vec4(moonPosition, 0.0)).xyz);
    lightWorldDir = normalize(viewPosToWorldPos(vec4(shadowLightPosition, 0.0)).xyz);

	isNoon = clamp01(dot(sunWorldDir, upWorldDir) * NOON_DURATION);
	isNight = clamp01(dot(moonWorldDir, upWorldDir) * NIGHT_DURATION);
	sunRiseSet = clamp01(1 - isNoon - isNight);

	sunColor = Transmittance1(vec3(0.0, camera_h, 0.0), sunWorldDir, 5.0);
	sunColor = mix(sunColor, vec3(getLuminance(sunColor)), rainStrength * 0.8);
	nightColor = isNight * Transmittance1(vec3(0.0, camera_h, 0.0), moonWorldDir, 5.0) * NIGHT_LIGHT_BRI;
	lightColor = sunColor + nightColor;
	ambiColor = AtmosphericScattering(upWorldDir, sunWorldDir, 5.0, 3.0)
			+ isNight * nightColor * NIGHT_AMBIENT_BRI;

	gl_Position = ftransform();
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
}
#endif