varying vec2 texcoord;

varying vec3 sunWorldDir, moonWorldDir, lightWorldDir;

varying vec3 sunColor, nightColor, lightColor, ambiColor;

varying float sunRiseSet, isNoon, isNight;

#include "/lib/uniform.glsl"
#include "/lib/setting.glsl"
#include "/lib/common/common.glsl"
#include "/lib/common/position.glsl"
#include "/lib/camera/colorToolkit.glsl"
#include "/lib/camera/filter.glsl"
#include "/lib/common/noise.glsl"
#include "/lib/common/normal.glsl"
#include "/lib/atmosphere/atmosphericScattering.glsl"
#include "/lib/atmosphere/celestial.glsl"


#ifdef FSH
// const bool colortex0MipmapEnabled = true;
const bool colortex4MipmapEnabled = true;

#define CPS_FSH
#include "/lib/TAA.glsl"
#include "/lib/common/materialIdMapper.glsl"
#include "/lib/atmosphere/volumetricClouds.glsl"
#include "/lib/water/waterEffect.glsl"


void main() {
	vec3 color = texture(colortex0, texcoord).rgb;

	float depth0 = texture(depthtex0, texcoord).r;
	float depth = texture(depthtex1, texcoord).r;
	float linearDepth0 = expDepthToLinearDepth(depth0);
	float linearDepth = expDepthToLinearDepth(depth);

	vec4 viewPos = screenPosToViewPos(vec4(texcoord, depth, 1.0));
    vec4 worldPos = viewPosToWorldPos(viewPos);
    vec3 mcPos = worldPos.xyz + cameraPosition;
    float worldDis = clamp(length(worldPos.xyz) / far, 0.0, 1.0);
    vec3 worldDir = normalize(worldPos.xyz);

	// TAA Jitter coords
	vec2 tTexcoord = unTAAJitterTexcoord(texcoord);
	vec4 tViewPos = screenPosToViewPos(vec4(tTexcoord, depth, 1.0));
	vec4 tWorldPos = viewPosToWorldPos(tViewPos);
	vec3 tWorldDir = normalize(tWorldPos.xyz);

	// block normal
	vec3 normal = normalize(normalDecode(texture(colortex1, texcoord).ba));
	vec3 normalW = normalize((gbufferModelViewInverse * vec4(normal, 0.0)).xyz);





	if(skyB > 0.5){
		// Cloud & Sun & Moon & Stars
		float transmittance = 1.0;
		color.rgb = getVolumetricClouds(color, depth, tWorldDir, normal, transmittance);
		color.rgb += drawCelestial(tWorldDir, transmittance);
	}





	vec3 waterViewNormal = vec3(0.0);
	waterViewNormal = normalize(normalDecode(texture(colortex7, texcoord).xy));
	vec3 waterScreenNormal = normalize(viewPosToScreenPos(vec4(waterViewNormal, 0.0)).xyz);
	vec3 waterWorldNormal = normalize(viewPosToWorldPos(vec4(waterViewNormal, 0.0)).xyz);	
	
	// 制造全内反射
	vec3 waterFogColorUsed = waterFogColor * 1.00 * getLuminance(ambiColor);
	if(waterA > 0.5){
		if(isEyeInWater == 1){
			vec3 refractDir = normalize(refract(-worldDir, waterWorldNormal, WATER_REFRACT_IOR));
			color = mix(color, vec3(0.0), 1.0 - clamp01(length(refractDir)));
			color *= eyeBrightnessSmooth.y / 240.0;
		}
		// color.rgb *= 1.0 + pow(mix(1.0, dot(waterWorldNormal, sunWorldDir), 0.4), 80.0) * 1.2;
	}
	// 水下体积雾
	if(isEyeInWater == 1){
		float worldDis0 = length(viewPosToWorldPos(screenPosToViewPos(vec4(texcoord, depth0, 1.0))));
		color = underWaterFog(color, worldDir, worldDis0, waterFogColorUsed);
	}


	color.rgb = max(BLACK, color.rgb);

/* DRAWBUFFERS:0 */
	gl_FragData[0] = vec4(color, 1.0);
}


#endif
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef VSH

void main() {
	sunWorldDir = normalize(viewPosToWorldPos(vec4(sunPosition, 0.0)).xyz);
    moonWorldDir = normalize(viewPosToWorldPos(vec4(moonPosition, 0.0)).xyz);
    lightWorldDir = normalize(viewPosToWorldPos(vec4(shadowLightPosition, 0.0)).xyz);

	isNoon = clamp01(dot(sunWorldDir, upWorldDir) * NOON_DURATION);
	isNight = clamp01(dot(moonWorldDir, upWorldDir) * NIGHT_DURATION);
	sunRiseSet = clamp01(1 - isNoon - isNight);

	sunColor = Transmittance1(vec3(0.0, camera_h, 0.0), sunWorldDir, 5.0);
	sunColor = mix(sunColor, vec3(getLuminance(sunColor)), rainStrength * 0.8);
	nightColor = isNight * Transmittance1(vec3(0.0, camera_h, 0.0), moonWorldDir, 5.0) * NIGHT_LIGHT_BRI;
	lightColor = sunColor + nightColor;
	ambiColor = AtmosphericScattering(upWorldDir, sunWorldDir, 5.0, 3.0)
			+ isNight * nightColor * NIGHT_AMBIENT_BRI;

	gl_Position = ftransform();
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
}
#endif