varying vec2 texcoord;

varying vec3 sunWorldDir, moonWorldDir, lightWorldDir;

varying vec3 sunColor, nightColor, lightColor, ambiColor;

varying float sunRiseSet, isNoon, isNight;

#include "/lib/uniform.glsl"
#include "/lib/setting.glsl"
#include "/lib/common/common.glsl"
#include "/lib/common/position.glsl"
#include "/lib/camera/colorToolkit.glsl"
#include "/lib/common/noise.glsl"
#include "/lib/common/normal.glsl"
#include "/lib/atmosphere/atmosphericScattering.glsl"
#include "/lib/atmosphere/celestial.glsl"

#ifdef FSH
const bool colortex0MipmapEnabled = true;
// const bool colortex4MipmapEnabled = true;

#define CPS_FSH
#include "/lib/lighting/mcLightmap.glsl"
#include "/lib/camera/filter.glsl"
// #include "/lib/camera/bloom.glsl"
#include "/lib/TAA.glsl"
#include "/lib/common/materialIdMapper.glsl"
// #include "/lib/water/waterNormal.glsl"
#include "/lib/water/waterReflectionRefraction.glsl"

void main() {
	vec3 color = texture(colortex0, texcoord).rgb;

	float depth0 = texture(depthtex0, texcoord).r;
	float depth = texture(depthtex1, texcoord).r;
	float linearDepth0 = expDepthToLinearDepth(depth0);
	float linearDepth = expDepthToLinearDepth(depth);

	vec4 viewPos = screenPosToViewPos(vec4(texcoord, depth, 1.0));
    vec4 worldPos = viewPosToWorldPos(viewPos);
    vec3 mcPos = worldPos.xyz + cameraPosition;
    float worldDis = clamp(length(worldPos.xyz) / far, 0.0, 1.0);
    vec3 worldDir = normalize(worldPos.xyz);



	// TAA Jitter coords
	vec2 tTexcoord = unTAAJitterTexcoord(texcoord);
	vec4 tViewPos = screenPosToViewPos(vec4(tTexcoord, depth, 1.0));
	vec4 tWorldPos = viewPosToWorldPos(tViewPos);
	vec3 tMcPos = tWorldPos.xyz + cameraPosition;
	vec3 tWorldDir = normalize(tWorldPos.xyz);

	vec3 waterViewNormal = vec3(0.0);
	waterViewNormal = normalize(normalDecode(texture(colortex7, texcoord).xy));
	vec3 waterScreenNormal = normalize(viewPosToScreenPos(vec4(waterViewNormal, 0.0)).xyz);
	vec3 waterWorldNormal = normalize(viewPosToWorldPos(vec4(waterViewNormal, 0.0)).xyz);

	float waterLightmap = unpack16Bit(texture(colortex7, texcoord).b).x;
	waterLightmap = AdjustLightmap(vec2(0.0, waterLightmap)).y;





	// 水面折射 & 反射
	if(waterA > 0.5){
		float worldDis0 = length(viewPosToWorldPos(screenPosToViewPos(vec4(texcoord, depth0, 1.0))));
		float worldDis1 = length(viewPosToWorldPos(screenPosToViewPos(vec4(texcoord, depth, 1.0))));

		// vec3 mcPos0 = viewPosToWorldPos(screenPosToViewPos(vec4(texcoord, depth0, 1.0))).xyz + cameraPosition;
		color.rgb = waterRefraction(color, waterViewNormal, worldDis0, worldDis1);
		// color.rgb += pow(mix(1.0, max(0.0, dot(waterWorldNormal, sunWorldDir)), 1.0), 60.0) * 10.0;
		vec3 startPos = screenPosToViewPos(vec4(texcoord, depth0, 1.0)).xyz;
		color.rgb = waterReflection(color, startPos, tWorldDir, waterViewNormal, waterWorldNormal, waterLightmap);
	}





	// translucency color
	vec4 translucencyColor = texture(colortex6, texcoord);
	gammaToLinear(translucencyColor.rgb);
	if(translucencyColor.a > 0.01 && waterA < 0.1){
		color.rgb = mix(color.rgb, translucencyColor.rgb * waterLightmap * getLuminance(lightColor) * 0.3 * (1.0 - rainStrength * 0.7), translucencyColor.a);
	}





	// TEST
	// color = pow(vec3(texture(shadowcolor1, texcoord).rgb), vec3(2.2));





	//BLOOM 亮度预计算， Lod 为2，代表从2级开始算起，0也并无差别
	// vec3 bloom = textureLod(colortex0, unTAAJitterTexcoord(texcoord), 2.0).rgb;
	// vec3 bloomLumThreshold = vec3(0.0);
	// bloom -= bloomLumThreshold;
	vec3 bloom = color;
	bloom = max(BLACK, bloom);





	color.rgb = max(BLACK, color.rgb);


/* DRAWBUFFERS:04 */
	gl_FragData[0] = vec4(color, 1.0);
	gl_FragData[1] = vec4(bloom, 1.0);
}


#endif
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef VSH

void main() {
	sunWorldDir = normalize(viewPosToWorldPos(vec4(sunPosition, 0.0)).xyz);
    moonWorldDir = normalize(viewPosToWorldPos(vec4(moonPosition, 0.0)).xyz);
    lightWorldDir = normalize(viewPosToWorldPos(vec4(shadowLightPosition, 0.0)).xyz);

	isNoon = clamp01(dot(sunWorldDir, upWorldDir) * NOON_DURATION);
	isNight = clamp01(dot(moonWorldDir, upWorldDir) * NIGHT_DURATION);
	sunRiseSet = clamp01(1 - isNoon - isNight);

	sunColor = Transmittance1(vec3(0.0, camera_h, 0.0), sunWorldDir, 5.0);
	sunColor = mix(sunColor, vec3(getLuminance(sunColor)), rainStrength * 0.8);
	nightColor = isNight * Transmittance1(vec3(0.0, camera_h, 0.0), moonWorldDir, 5.0) * NIGHT_LIGHT_BRI;
	lightColor = sunColor + nightColor;
	ambiColor = AtmosphericScattering(upWorldDir, sunWorldDir, 5.0, 3.0)
			+ isNight * nightColor * NIGHT_AMBIENT_BRI;
	

	gl_Position = ftransform();
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
}
#endif