//varying
varying vec2 lmcoord;
varying vec2 texcoord;
varying vec4 glcolor;
varying vec2 normal;

varying float blockID;

varying vec4 vViewPos, vMcPos;
varying mat3 tbnMatrix, T_tbnMatrix;

varying vec4 vTexCoordAM;
varying vec2 vTexCoord;
varying vec2 textureResolution1;

#include "/lib/uniform.glsl"
#include "/lib/setting.glsl"
#include "/lib/common/common.glsl"
#include "/lib/common/position.glsl"
#include "/lib/common/normal.glsl"

#ifdef FSH
// flat in vec2 midMinTexcoord;
flat in vec4 v_tcrange;

#include "/lib/surface/parallaxMapping.glsl"
#include "/lib/anisotropicFiltering.glsl"

const bool colortex0MipmapEnabled = true;

void main() {
	vec3 normalO = normalize(normalDecode(normal));

	vec2 texGradX = dFdx(texcoord);
	vec2 texGradY = dFdy(texcoord);

	vec2 newTexCoord = vTexCoord * vTexCoordAM.pq + vTexCoordAM.st;
	vec2 parallaxUV = newTexCoord;
	vec3 parallaxOffset = vec3(0.0, 0.0, 1.0);  	// 用于偏移视差阴影的起始点
	vec3 parallaxOffset1 = vec3(0.0, 0.0, 1.0);		// 用于偏移各向异性过滤采样
	float shadow = 1.0;

#ifdef PARALLAX_MAPPING
	vec3 viewDirTS = normalize(T_tbnMatrix * vViewPos.xyz);
	// ivec2 atlasTiles = atlasSize;
	// float atlasAspectRatio = atlasTiles.x / atlasTiles.y;
	// viewDirTS.y *= atlasAspectRatio;
	// viewDirTS = normalize(viewDirTS);
	vec3 lightDirTS = normalize(T_tbnMatrix * shadowLightPosition);
	parallaxUV = parallaxMapping(parallaxUV, viewDirTS, lightDirTS, texGradX, texGradY, parallaxOffset, parallaxOffset1);


	#ifdef PARALLAX_SHADOW
	if(parallaxOffset.z < 1.0){
		shadow = parallaxShadow(parallaxOffset, lightDirTS, texGradX, texGradY);
		// 法线与光源垂直时shadow为1.0，不垂直时为原数值
		shadow = max(clamp01(1.0 - abs(dot(normalize(shadowLightPosition), normalO))), shadow);
	}
	#endif

#endif

	// vec4 text = textureGrad(tex, parallaxUV, texGradX, texGradY);
	// normalO = normalize((gbufferModelViewInverse * vec4(normalO, 0.0)).xyz);
	// if(normalO.x > 0.9 || normalO.y > 0.9 || normalO.z > 0.9){
		// vec4 text = textureGrad(tex, parallaxUV, texGradX, texGradY);
		vec4 text = textureAniso2(tex, parallaxUV, parallaxOffset1.st);
	// }
	
	text.rgb *= mix(1.0, shadow, 0.40);
	

	vec2 normalTex = normalEncode(normalize(tbnMatrix * (textureGrad(normals, parallaxUV, texGradX, texGradY).rgb * 2.0 - 1.0)));
	float specularTex = packRGBToFloat(texture(specular, parallaxUV).rgb);

#ifdef VOXEL_PARALLAX
    normalTex = normal;
#endif

	vec4 color = text * glcolor;

/* DRAWBUFFERS:014 */
	gl_FragData[0] = color;
	gl_FragData[1] = vec4(pack2x8Bits(vec2(blockID / BLOCK_SCALE, GBF_TERRAIN / BLOCK_SCALE)), pack2x8Bits(lmcoord), normalTex);
	gl_FragData[2] = vec4(0.0, 0.0, 0.0, shadow);
	// gl_FragData[3].g = specularTex;
}

#endif
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef VSH
#include "/lib/TAA.glsl"
// #include "/lib/camera/filter.glsl"
// flat out vec2 midMinTexcoord;
flat out vec4 v_tcrange;

attribute vec4 mc_Entity;
attribute vec4 mc_midTexCoord;

attribute vec4 at_tangent;

#define GBF_VSH
#include "/lib/common/materialIdMapper.glsl"
#include "/lib/wavingPlants.glsl"

void main() {
	blockID = IDMapping();
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
	lmcoord  = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;


	// 坐标
	vViewPos = gl_ModelViewMatrix * gl_Vertex;
	vec4 vWorldPos = viewPosToWorldPos(vViewPos);
	vMcPos = vec4(vWorldPos.xyz + cameraPosition, 1.0);
	float vWorldDis = length(vWorldPos.xyz);

	// TBN Mat 参考了 BSL shader
	vec3 N = normalize(gl_NormalMatrix * gl_Normal);
	vec3 B = normalize(gl_NormalMatrix * cross(at_tangent.xyz, gl_Normal.xyz) * at_tangent.w);
	vec3 T = normalize(gl_NormalMatrix * at_tangent.xyz);
	tbnMatrix = mat3(T, B, N);
	T_tbnMatrix = transpose(tbnMatrix);

	// 偏移钳制参考了 BSL shader
	vec2 midCoord = (gl_TextureMatrix[0] *  mc_midTexCoord).st;
	vec2 texMinMidCoord = texcoord - midCoord;

	// midMinTexcoord = (midCoord - texcoord) * 2;


	vTexCoordAM.pq  = abs(texMinMidCoord) * 2;
	vTexCoordAM.st  = min(texcoord, midCoord - texMinMidCoord);
	vTexCoord.xy    = sign(texMinMidCoord) * 0.5 + 0.5;

	ivec2 textureResolution = ivec2(round(abs(gl_MultiTexCoord0.xy - mc_midTexCoord.xy) * atlasSize * 2.0));
    v_tcrange.zw = textureResolution / vec2(atlasSize);
    v_tcrange.xy = mc_midTexCoord.xy - v_tcrange.zw * 0.5;
	textureResolution1 = textureResolution;

	vec3 normalW = gl_NormalMatrix * gl_Normal;
	normal = normalEncode(normalW);

	if(vWorldDis <  WAVING_DISTANCE){	
		// 植物晃动所使用的法线
		// normalW = normalize(gbufferProjectionInverse * vec4(normalW, 0.0)).rgb;
		// if(dot(normalW.xz, -wind_dir) < 0.0) normalW = -normalW;
		const float waving_rate = WAVING_RATE;
		// 植物晃动
		if(blockID == PLANTS_SHORT && gl_MultiTexCoord0.t < mc_midTexCoord.t){
			// pos, normal, A, B, D_amount, y_waving_amount
			vMcPos.xyz = wavingPlants(vMcPos.xyz, normalW, PLANTS_SHORT_AMPLITUDE, waving_rate, 0.0, 0.0);
		}

		if(blockID == LEAVES){
			vMcPos.xyz = wavingPlants(vMcPos.xyz, normalW, LEAVES_AMPLITUDE, waving_rate, 0.0, 1.0);
		}

		if((blockID == PLANTS_TALL_L && gl_MultiTexCoord0.t < mc_midTexCoord.t) || blockID == PLANTS_TALL_U){
			vMcPos.xyz = wavingPlants(vMcPos.xyz, normalW, PLANTS_TALL_AMPLITUDE, waving_rate, 0.0, 0.0);
		}
	}


	gl_Position = gl_ProjectionMatrix * gbufferModelView * vec4(vMcPos.xyz - cameraPosition, 1.0);
	
	vec2 jitter = Halton_2_3[frameCounter % 8];	//-1 to 1
	jitter /= vec2(viewWidth, viewHeight);
	gl_Position.xyz /= gl_Position.w;
    gl_Position.xy += jitter * TAA_JITTER_AMOUNT;
    gl_Position.xyz *= gl_Position.w;
	
	glcolor = gl_Color;
	
}
#endif