//varying
varying vec2 lmcoord;
varying vec2 texcoord;
varying vec4 glcolor;
varying vec2 normal;
varying vec4 vViewPos, vMcPos;
varying mat3 tbnMatrix, T_tbnMatrix;
varying float worldDis;


#include "/lib/uniform.glsl"
#include "/lib/setting.glsl"
#include "/lib/common/common.glsl"
#include "/lib/common/normal.glsl"
#include "/lib/common/position.glsl"
#include "/lib/camera/filter.glsl"

#ifdef FSH
flat in float translucencyID;
flat in float isIce, isWater;

#include "/lib/water/waterNormal.glsl"

void main() {
	vec4 colorT = texture(tex, texcoord) * glcolor;
	vec4 mcPos = vMcPos;
	vec2 waveNormal;

	if(isWater > 0.5){
		vec3 viewDirTS = normalize(T_tbnMatrix * vViewPos.xyz);
		vec2 waveParallaxUV = waveParallaxMapping(mcPos.xz, viewDirTS);
		waveNormal = normalEncode(normalize(tbnMatrix * getWaveNormal(waveParallaxUV)));
		colorT.a = 0.0;
	}else{
		waveNormal = normal;
	}


	
	
/* DRAWBUFFERS:0367 */
	gl_FragData[0] = vec4(colorT.rgb, 0.0);
	gl_FragData[1] = vec4(normal, 0.0, 1.0);
	gl_FragData[2] = colorT;
	gl_FragData[3] = vec4(waveNormal, pack2x8Bits(vec2(lmcoord.y, isWater)), 1.0);
}

#endif
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef VSH
flat out float translucencyID;
flat out float isIce, isWater;
attribute vec4 mc_Entity;
attribute vec4 at_tangent;

#define GBF_VSH
#include "/lib/TAA.glsl"
#include "/lib/common/materialIdMapper.glsl"

void main() {
	translucencyID = IDMappingT();
	isWater = checkInRange(mc_Entity.x, 8.0, 0.3);
	isIce = mc_Entity.x == 79 ? 1.0 : 0.0;

	vViewPos = gl_ModelViewMatrix * gl_Vertex;
	vec4 vWorldPos = viewPosToWorldPos(vViewPos);
	vMcPos = vec4(vWorldPos.xyz + cameraPosition, 1.0);
	worldDis = length(vWorldPos.xyz);

	// TBN Mat 参考了 BSL shader
	vec3 N = normalize(gl_NormalMatrix * gl_Normal);
	vec3 B = normalize(gl_NormalMatrix * cross(at_tangent.xyz, gl_Normal.xyz) * at_tangent.w);
	vec3 T = normalize(gl_NormalMatrix * at_tangent.xyz);
	tbnMatrix = mat3(T, B, N);
	T_tbnMatrix = transpose(tbnMatrix);

	gl_Position = ftransform();
	// vec2 jitter = Halton_2_3[frameCounter % 8];	//-1 to 1
	// jitter /= vec2(viewWidth, viewHeight);

	// gl_Position.xyz /= gl_Position.w;
    // gl_Position.xy += jitter * TAA_JITTER_AMOUNT;
    // gl_Position.xyz *= gl_Position.w;
	
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
	lmcoord  = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
	glcolor = gl_Color;
	normal = normalEncode(gl_NormalMatrix * gl_Normal);
}
#endif