varying vec4 texcoord;
varying vec4 glColor;
varying vec3 normal;
varying float dist;

#include "/lib/uniform.glsl"
#include "/lib/setting.glsl"
#include "/lib/common/common.glsl"
#include "/lib/common/position.glsl"

#ifdef FSH
flat in float isPlants;

void main(){
    vec4 color = texture(tex, texcoord.st) * glColor;
    vec3 shadowNormal = normal;
    if(isPlants > 0.5){
        shadowNormal = (shadowModelView * vec4(upWorldDir, 0.0)).xyz;
    }
    
    gl_FragData[0] = color;
    gl_FragData[1] = vec4(shadowNormal * 0.5 + 0.5, dist / 1.41421356237);
}
#endif

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef VSH
flat out float isPlants;

attribute vec4 mc_Entity;
attribute vec4 mc_midTexCoord;

#define GBF_VSH
#include "/lib/common/materialIdMapper.glsl"
#include "/lib/wavingPlants.glsl"


void main(){
    float blockID = IDMapping();
    float translucencyID = IDMappingT();

    vec4 vWorldPos = shadowModelViewInverse * shadowProjectionInverse * ftransform();
    float vWorldDis = length(vWorldPos.xyz);
    vec4 vMcPos = vec4(vWorldPos.xyz + cameraPosition, 1.0);

    isPlants = 0.0;
    if(vWorldDis < WAVING_DISTANCE){	
		vec3 normalW = vec3(0.0);
		const float waving_rate = WAVING_RATE;
		// 植物晃动
		if(blockID == PLANTS_SHORT && gl_MultiTexCoord0.t < mc_midTexCoord.t){
			// pos, normal, A, B, D_amount, y_waving_amount
			vMcPos.xyz = wavingPlants(vMcPos.xyz, normalW, PLANTS_SHORT_AMPLITUDE, waving_rate, 0.0, 0.0);
		}

		if(blockID == LEAVES){
			vMcPos.xyz = wavingPlants(vMcPos.xyz, normalW, LEAVES_AMPLITUDE, waving_rate, 0.0, 1.0);
		}

		if((blockID == PLANTS_TALL_L && gl_MultiTexCoord0.t < mc_midTexCoord.t) || blockID == PLANTS_TALL_U){
			vMcPos.xyz = wavingPlants(vMcPos.xyz, normalW, PLANTS_TALL_AMPLITUDE, waving_rate, 0.0, 0.0);
		}
	}
    

    // 水面剔除
    if(translucencyID == WATER){
        vMcPos.xyz += 10000.0;
    }

    gl_Position = shadowProjection * shadowModelView * vec4(vMcPos.xyz - cameraPosition, 1.0);

    dist = length(gl_Position.xy);
    float distortFactor = (1.0 - shadowMap_BIAS) + dist * shadowMap_BIAS;
    gl_Position.xy /= distortFactor;
    gl_Position.z = mix(gl_Position.z, 0.5, 0.8);

    texcoord = gl_TextureMatrix[0] * gl_MultiTexCoord0;
    glColor = gl_Color;
    normal = normalize(gl_NormalMatrix * gl_Normal);
}
#endif